@@ -1,6 +1,8 @@
 lgi = require('lgi')
 kdb = lgi.require('GElektra')
 
+TEST_NS = "user/tests/gi_lua"
+
 -- kdbconfig.h
 assert(type(kdb.DB_SYSTEM) == "string")
 assert(type(kdb.DB_USER)   == "string")
@@ -15,13 +17,9 @@ assert(type(kdb.VERSION_MINOR) == "number")
 assert(type(kdb.VERSION_MICRO) == "number")
 assert(kdb.KS_END == nil)
 
---[[
--- disabled until elektra libraries can be loaded by
--- the build system (and static build is gone)
-
 -- ctor
 assert(kdb.KDB:is_type_of(kdb.KDB()))
-error = kdb.Key()
+local error = kdb.Key()
 assert(kdb.KDB:is_type_of(kdb.KDB(error)))
 
 -- get
@@ -30,30 +28,38 @@ do
 	local ks = kdb.KeySet(100)
 	db:get(ks, "system/elektra")
 
-	key = ks:lookup("system/elektra/version/constants/KDB_VERSION")
+	local key = ks:lookup("system/elektra/version/constants/KDB_VERSION")
 	assert(key.value == kdb.VERSION)
 end
 
 -- set
 do
 	local db = kdb.KDB()
-	ks = kdb.KeySet(100)
-	db:get(ks, "user/MyApp")
+	local ks = kdb.KeySet(100)
+	db:get(ks, TEST_NS)
 
-	key = ks:lookup("user/MyApp/mykey")
+	local key = ks:lookup(TEST_NS .. "/mykey")
 	if not key then
-		key = kdb.Key("user/MyApp/mykey")
+		key = kdb.Key(TEST_NS .. "/mykey")
 		ks:append(key)
 	end
 	key.string = "new_value"
 
-	db:set(ks, "user/MyApp")
+	db:set(ks, TEST_NS)
 end
 
 do
 	local db = kdb.KDB()
-	ks = kdb.KeySet(100)
-	db:get(ks, "user/MyApp")
-	assert(ks:lookup("user/MyApp/mykey").value == "new_value")
+	local ks = kdb.KeySet(100)
+	db:get(ks, TEST_NS)
+	assert(ks:lookup(TEST_NS .. "/mykey").value == "new_value")
+end
+
+-- cleanup
+do
+	local db = kdb.KDB()
+	local ks = kdb.KeySet(100)
+	db:get(ks, TEST_NS)
+	ks:cut(kdb.Key(TEST_NS))
+	db:set(ks, TEST_NS)
 end
-]]--
