@@ -44,6 +44,27 @@
 
 #include <kdbinternal.h>
 
+/**
+ * @brief Allocate a backend
+ *
+ * Initialize everything with zero, except: sizes with -1
+ * and refcounter with 1
+ *
+ * @return 
+ */
+static Backend* elektraBackendAllocate()
+{
+	Backend *backend = elektraCalloc(sizeof(struct _Backend));
+
+	backend->refcounter = 1;
+
+	backend->specsize = -1;
+	backend->dirsize = -1;
+	backend->usersize = -1;
+	backend->systemsize = -1;
+	return backend;
+}
+
 
 /**Builds a backend out of the configuration supplied
  * from:
@@ -90,8 +111,7 @@ Backend* elektraBackendOpen(KeySet *elektraConfig, KeySet *modules, Key *errorKe
 
 	root = ksNext (elektraConfig);
 
-	Backend *backend = elektraCalloc(sizeof(struct _Backend));
-	backend->refcounter = 1;
+	Backend *backend = elektraBackendAllocate();
 
 	while ((cur = ksNext(elektraConfig)) != 0)
 	{
@@ -178,8 +198,7 @@ Backend* elektraBackendOpen(KeySet *elektraConfig, KeySet *modules, Key *errorKe
  */
 Backend* elektraBackendOpenMissing(Key *mp)
 {
-	Backend *backend = elektraCalloc(sizeof(struct _Backend));
-	backend->refcounter = 1;
+	Backend *backend = elektraBackendAllocate();
 
 	Plugin *plugin = elektraPluginMissing();
 	if (!plugin)
@@ -201,7 +220,8 @@ Backend* elektraBackendOpenMissing(Key *mp)
 }
 
 /**
- * Opens a default backend using the plugin named dump and resolver.
+ * Opens a default backend using the plugin named KDB_DEFAULT_RESOLVER
+ * and KDB_DEFAULT_STORAGE.
  *
  * @param modules the modules to work with
  * @param errorKey the key to issue warnings and errors to
@@ -209,8 +229,7 @@ Backend* elektraBackendOpenMissing(Key *mp)
  */
 Backend* elektraBackendOpenDefault(KeySet *modules, Key *errorKey)
 {
-	Backend *backend = elektraCalloc(sizeof(struct _Backend));
-	backend->refcounter = 1;
+	Backend *backend = elektraBackendAllocate();
 
 	KeySet *resolverConfig = ksNew(5,
 		keyNew("system/path", KEY_VALUE, KDB_DB_FILE, KEY_END),
@@ -263,8 +282,7 @@ Backend* elektraBackendOpenDefault(KeySet *modules, Key *errorKey)
  */
 Backend* elektraBackendOpenModules(KeySet *modules, Key *errorKey)
 {
-	Backend *backend = elektraCalloc(sizeof(struct _Backend));
-	backend->refcounter = 1;
+	Backend *backend = elektraBackendAllocate();
 
 	cursor_t save = ksGetCursor (modules);
 	KeySet *defaultConfig = ksNew(5,
@@ -303,8 +321,7 @@ Backend* elektraBackendOpenModules(KeySet *modules, Key *errorKey)
  */
 Backend* elektraBackendOpenVersion(Key * errorKey ELEKTRA_UNUSED)
 {
-	Backend *backend = elektraCalloc(sizeof(struct _Backend));
-	backend->refcounter = 1;
+	Backend *backend = elektraBackendAllocate();
 
 	Plugin *plugin = elektraPluginVersion();
 	if (!plugin)
@@ -326,6 +343,52 @@ Backend* elektraBackendOpenVersion(Key * errorKey ELEKTRA_UNUSED)
 	return backend;
 }
 
+
+/**
+ * @brief Update internal size in backend
+ *
+ * @param backend the backend to update
+ * @param parent for parent
+ * @param size to update (-1 default, 0 empty, >0 otherwise)
+ *
+ * @pre parent must be serializable namespace
+ *
+ * @retval -1 if invalid parent (assert)
+ * @retval 0 on success
+ */
+int elektraBackendUpdateSize(Backend *backend, Key *parent, int size)
+{
+	switch (keyGetNamespace(parent))
+	{
+	case KEY_NS_SPEC:
+		backend->specsize = size;
+		break;
+	case KEY_NS_DIR:
+		backend->dirsize = size;
+		break;
+	case KEY_NS_USER:
+		backend->usersize = size;
+		break;
+	case KEY_NS_SYSTEM:
+		backend->systemsize = size;
+		break;
+	case KEY_NS_PROC:
+	case KEY_NS_EMPTY:
+	case KEY_NS_META:
+	case KEY_NS_CASCADING:
+	case KEY_NS_NONE:
+		ELEKTRA_ASSERT(0 && "invalid namespace");
+		return -1;
+	}
+
+#if DEBUG && VERBOSE
+	printf ("user: %d\n", backend->usersize);
+	printf ("system: %d\n", backend->systemsize);
+#endif
+
+	return 0;
+}
+
 int elektraBackendClose(Backend *backend, Key* errorKey)
 {
 	int ret = 0;
