@@ -24,14 +24,67 @@
  * @endcode
  *
  * The kdb*() methods are used to access the storage, to get and set
- * @link keyset KeySets @endlink.
+ * @link keyset KeySets@endlink.
  *
- * They use some backend implementation to know the details about how
- * to access the storage.
+ * Parameters common for all these functions are:
  *
+ * - *handle*, as returned by kdbOpen(), need to be passed to every call
+ * - *parentKey* is used for every call to add warnings and set an
+ *   error. For kdbGet() / kdbSet() it is used to give an hint which keys
+ *   should be retrieved/stored.
+ *
+ * @note The parentKey is an obligation for you, but only an hint for KDB.
+ * KDB does not remember anything
+ * about the configuration. You need to pass the same configuration
+ * back to kdbSet(), otherwise parts of the configuration get
+ * lost. Only keys below the parentKey are subject for change, the rest
+ * must be left untouched.
+ *
+ * KDB uses different backend implementations that know the details
+ * about how to access the storage.
  * One backend consists of multiple plugins.
  * See @link plugin writing a new plugin @endlink for information
  * about how to write a plugin.
+ * Backends are state-less regarding the configuration (because of that
+ * you must pass back the whole configuration for every backend), but
+ * have a state for:
+ *
+ * - a two phase-commit
+ * - a conflict detection (error 30) and
+ * - optimizations that avoid redoing already done operations.
+ *
+ * @image html state.png "State"
+ * @image latex state.png "State"
+ *
+ * As we see in the figure, kdbOpen() can be called arbitrarily often in any
+ * number of threads.
+ *
+ * For every handle you got from kdbOpen(), for every parentKey with a
+ * different name, *only* the shown state transitions
+ * are valid. From a freshly opened KDB, only kdbGet() and kdbClose()
+ * are allowed, because otherwise conflicts (error 30) would not be detected.
+ *
+ * Once kdbGet() was called (for a specific handle+parentKey),
+ * any number of kdbGet() and kdbSet() can be
+ * used with this handle respective parentKey, unless kdbSet() had
+ * a conflict (error 30) with another application.
+ * Every affair with KDB needs to be finished with kdbClose().
+ *
+ * The name of the parentKey in kdbOpen() and kdbClose() does not matter.
+ *
+ * In the usual case we just have one parentKey and one handle. In
+ * these cases we just have to remember to use kdbGet() before
+ * kdbSet():
+ *
+ * @include kdbintro.c
+ *
+ * To output warnings, you can use following code:
+ *
+ * @snippet tests.c warnings
+ *
+ * To output the error, you can use following code:
+ *
+ * @snippet tests.c error
  *
  * @{
  */
@@ -69,12 +122,18 @@
 #include <stdio.h>
 #endif
 
+#ifdef HAVE_ERRNO_H
+#include <errno.h>
+#endif
+
 #include <kdbinternal.h>
 
 
 /**
  * @brief Opens the session with the Key database.
  *
+ * @pre errorKey must be a valid key, e.g. created with keyNew()
+ *
  * The method will bootstrap itself the following way.
  * The first step is to open the default backend. With it
  * system/elektra/mountpoints will be loaded and all needed
@@ -85,7 +144,6 @@
  * You must always call this method before retrieving or committing any
  * keys to the database. In the end of the program,
  * after using the key database, you must not forget to kdbClose().
- * You can use the atexit () handler for it.
  *
  * The pointer to the @p KDB structure returned will be initialized
  * like described above, and it must be passed along on any kdb*()
@@ -93,22 +151,8 @@
  *
  * Get a @p KDB handle for every thread using elektra. Don't share the
  * handle across threads, and also not the pointer accessing it:
- * @code
-thread1
-{
-	KDB * h;
-	h = kdbOpen(parent);
-	// fetch keys and work with them
-	kdbClose(h, parent);
-}
-thread2
-{
-	KDB * h;
-	h = kdbOpen(parent);
-	// fetch keys and work with them
-	kdbClose(h, parent);
-}
- * @endcode
+ *
+ * @snippet kdbopen.c open
  *
  * You don't need kdbOpen() if you only want to
  * manipulate plain in-memory Key or KeySet objects.
@@ -131,6 +175,7 @@ KDB * kdbOpen(Key *errorKey)
 		return 0;
 	}
 
+	int errnosave = errno;
 	handle = elektraCalloc(sizeof(struct _KDB));
 
 	handle->modules = ksNew(0, KS_END);
@@ -140,6 +185,7 @@ KDB * kdbOpen(Key *errorKey)
 		elektraFree(handle);
 		ELEKTRA_SET_ERROR(94, errorKey,
 				"elektraModulesInit returned with -1");
+		errno = errnosave;
 		return 0;
 	}
 
@@ -151,6 +197,7 @@ KDB * kdbOpen(Key *errorKey)
 		elektraFree(handle);
 		ELEKTRA_SET_ERROR(40, errorKey,
 				"could not open default backend");
+		errno = errnosave;
 		return 0;
 	}
 
@@ -175,6 +222,7 @@ KDB * kdbOpen(Key *errorKey)
 
 		keySetName(errorKey, keyName(initialParent));
 		keyDel(initialParent);
+		errno = errnosave;
 		return handle;
 	}
 	keySetName(errorKey, keyName(initialParent));
@@ -210,6 +258,7 @@ KDB * kdbOpen(Key *errorKey)
 				"could not reopen and mount default backend");
 		kdbClose(handle, errorKey);
 		keyDel (initialParent);
+		errno = errnosave;
 		return 0;
 	}
 
@@ -221,7 +270,10 @@ KDB * kdbOpen(Key *errorKey)
 
 	elektraMountVersion (handle, errorKey);
 
+	// elektraSplitOpen(handle->split);
+
 	keyDel (initialParent);
+	errno = errnosave;
 	return handle;
 }
 
@@ -229,23 +281,22 @@ KDB * kdbOpen(Key *errorKey)
 /**
  * Closes the session with the Key database.
  *
- * This is the counterpart of kdbOpen().
- *
- * You must call this method when you finished your affairs with the key
- * database. You can manipulate Key and KeySet objects also after
- * kdbClose(), but you must not use any kdb*() call afterwards. You
- * can run kdbClose() in the atexit() handler.
- *
- * The @p handle parameter will be finalized and all resources associated to it
- * will be freed. After a kdbClose(), the @p handle can't be used anymore.
- *
  * @pre The handle must be a valid handle as returned from kdbOpen()
  *
  * @pre errorKey must be a valid key, e.g. created with keyNew()
  *
+ * This is the counterpart of kdbOpen().
+ *
+ * You must call this method when you finished your affairs with the key
+ * database. You can manipulate Key and KeySet objects also after
+ * kdbClose(), but you must not use any kdb*() call afterwards.
+ *
+ * The @p handle parameter will be finalized and all resources associated to it
+ * will be freed. After a kdbClose(), the @p handle cannot be used anymore.
+ *
  * @param handle contains internal information of
  *               @link kdbOpen() opened @endlink key database
- * @param errorKey the key which holds error information
+ * @param errorKey the key which holds error/warning information
  * @retval 0 on success
  * @retval -1 on NULL pointer
  * @ingroup kdb
@@ -257,6 +308,7 @@ int kdbClose(KDB *handle, Key *errorKey)
 		return -1;
 	}
 
+	int errnosave = errno;
 	elektraSplitDel (handle->split);
 
 	elektraTrieClose(handle->trie, errorKey);
@@ -277,6 +329,7 @@ int kdbClose(KDB *handle, Key *errorKey)
 
 	elektraFree(handle);
 
+	errno = errnosave;
 	return 0;
 }
 
@@ -296,6 +349,7 @@ static int elektraGetCheckUpdateNeeded(Split *split, Key *parentKey)
 	{
 		int ret = 0;
 		Backend *backend = split->handles[i];
+		clear_bit(split->syncbits[i], 1);
 
 		if (backend->getplugins[0])
 		{
@@ -309,19 +363,26 @@ static int elektraGetCheckUpdateNeeded(Split *split, Key *parentKey)
 			// store resolved filename
 			keySetString(split->parents[i],
 					keyString(parentKey));
+			// no keys in that backend
+			elektraBackendUpdateSize(backend, split->parents[i], 0);
 		}
-		if (ret == -1)
+		switch (ret)
 		{
+		case 1:
+			// Seems like we need to sync that
+			set_bit(split->syncbits[i], SPLIT_FLAG_SYNC);
+			++ updateNeededOccurred;
+			break;
+		case 0:
+			// Nothing to do here
+			break;
+		default:
+			ELEKTRA_ASSERT(0 && "resolver did not return 1 0 -1");
+		case -1:
 			// Ohh, an error occurred, lets stop the
 			// process.
 			return -1;
 		}
-		if (ret == 1)
-		{
-			/* Seems like we need to sync that */
-			split->syncbits[i] |= 1;
-			++ updateNeededOccurred;
-		}
 	}
 	return updateNeededOccurred;
 }
@@ -335,8 +396,14 @@ static int elektraGetCheckUpdateNeeded(Split *split, Key *parentKey)
  */
 static int elektraGetDoUpdate(Split *split, Key *parentKey)
 {
-	for (size_t i=0; i<split->size-1;i++)
+	const int bypassedSplits = 1;
+	for (size_t i=0; i<split->size-bypassedSplits;i++)
 	{
+		if (!test_bit(split->syncbits[i], SPLIT_FLAG_SYNC))
+		{
+			// skip it, update is not needed
+			continue;
+		}
 		Backend *backend = split->handles[i];
 		ksRewind (split->keysets[i]);
 		keySetName (parentKey, keyName(split->parents[i]));
@@ -365,15 +432,22 @@ static int elektraGetDoUpdate(Split *split, Key *parentKey)
 }
 
 
+
 /**
  * @brief Retrieve keys in an atomic and universal way.
  *
- * @pre kdbOpen() must be called before using this method.
+ * @pre The @p handle must be passed as returned from kdbOpen()
  *
  * @pre The @p returned KeySet must be a valid KeySet, e.g. constructed
  *     with ksNew().
  *
- * The @p returned KeySet may already contain some keys from previous
+ * @pre The @p parentKey Key must be a valid Key, e.g. constructed with
+ *     keyNew().
+ *
+ * If you pass NULL, which violates the preconditions,
+ * on any parameter kdbGet() will fail immediately without doing anything.
+ *
+ * The @p returned KeySet may already contain some keys, e.g. from previous
  * kdbGet() calls. The new retrieved keys will be appended using
  * ksAppendKey().
  *
@@ -393,61 +467,39 @@ static int elektraGetDoUpdate(Split *split, Key *parentKey)
  *     Make sure to not touch or remove keys outside the keys of interest,
  *     because others may need them!
  *
- * @see ksCut() for further remarks on that topic.
- *
  * @par Example:
- *
  * This example demonstrates the typical usecase within an application
  * (without error handling).
  *
- * @code
-KeySet *myConfig = ksNew(0,KS_END);
-Key *pkey = keyNew("system/sw/MyApp",KEY_END);
-KDB *handle = kdbOpen(pkey);
-
-kdbGet(handle, myConfig, pkey); // get the config initially
-
-Key *key = ksLookupByName(myConfig,"/sw/MyApp/key", 0);
-// check if key is not 0 and work with it...
-
-kdbGet(handle, myConfig, pkey); // get the config again
-
-ksDel (myConfig); // delete the in-memory configuration
-
-kdbClose(handle, pkey); // no more affairs with the key database.
-keyDel(pkey);
- * @endcode
- *
- * @par Details:
- *
- * If you pass NULL on any parameter kdbGet() will fail
- * immediately without doing anything.
+ * @include kdbget.c
  *
  * When a backend fails kdbGet() will return -1 with all
  * error and warning information in the @p parentKey.
  * The parameter @p returned will not be changed.
  *
  * @par Updates:
- *
- * In the first run of kdbGet all keys are retrieved. On subsequent
+ * In the first run of kdbGet all requested (or more) keys are retrieved. On subsequent
  * calls only the keys are retrieved where something was changed
  * inside the key database. The other keys stay unchanged in the
- * keyset, even when they were manipulated.
+ * keyset, even if they were manipulated.
  *
  * It is your responsibility to save the original keyset if you
  * need it afterwards.
  *
- * If you must get the same keyset again, e.g. in another
- * thread you need to open a second handle to the key database
- * using kdbOpen().
+ * If you want to get the same keyset again, you need to open a
+ * second handle to the key database using kdbOpen().
  *
  * @param handle contains internal information of @link kdbOpen() opened @endlink key database
- * @param parentKey parent key holds the information which keys should
- * 	be get (it is possible that more are retrieved) - an invalid name gets all keys
+ * @param parentKey is used to add warnings and set an error
+ *         information. Additionally, its name is an hint which keys
+ *         should be retrieved (it is possible that more are retrieved).
+ *           - cascading keys (starting with /) will retrieve the same path in all namespaces
+ *           - / will retrieve all keys
  * @param ks the (pre-initialized) KeySet returned with all keys found
  * 	will not be changed on error or if no update is required
  * @see ksLookup(), ksLookupByName() for powerful
  * 	lookups after the KeySet was retrieved
+ * @see kdbOpen() which needs to be called before
  * @see kdbSet() to save the configuration afterwards and kdbClose() to
  * 	finish affairs with the key database.
  * @retval 1 if the keys were retrieved successfully
@@ -465,44 +517,21 @@ int kdbGet(KDB *handle, KeySet *ks, Key *parentKey)
 
 	if (ns == KEY_NS_META)
 	{
-		ELEKTRA_SET_ERROR(104, parentKey,
-				"invalid key name passed to kdbGet");
+		ELEKTRA_SET_ERRORF(104, parentKey,
+				"metakey with name \"%s\" passed to kdbGet",
+				keyName(parentKey));
 		return -1;
 	}
 
-	Key *initialParent = keyDup (parentKey);
-
-	if (ns == KEY_NS_CASCADING)
+	if (ns == KEY_NS_EMPTY)
 	{
-		keySetName(parentKey, "user");
-		keyAddName(parentKey, keyName(initialParent));
-		if (kdbGet(handle, ks, parentKey) == -1)
-		{
-			elektraKeySetName(parentKey,
-					keyName(initialParent),
-					KEY_CASCADING_NAME);
-			keyDel(initialParent);
-			return -1;
-		}
-
-		keySetName(parentKey, "system");
-		keyAddName(parentKey, keyName(initialParent));
-		if (kdbGet(handle, ks, parentKey) == -1)
-		{
-			elektraKeySetName(parentKey,
-					keyName(initialParent),
-					KEY_CASCADING_NAME);
-			keyDel(initialParent);
-			return -1;
-		}
-
-		elektraKeySetName(parentKey,
-				keyName(initialParent),
-				KEY_CASCADING_NAME);
-		keyDel(initialParent);
-		return 1;
+		ELEKTRA_ADD_WARNING(105, parentKey,
+				"invalid key name passed to kdbGet");
 	}
 
+	int errnosave = errno;
+	Key *initialParent = keyDup (parentKey);
+
 #if DEBUG && VERBOSE
 	fprintf (stderr, "now in new kdbGet (%s)\n",
 			keyName(parentKey));
@@ -529,13 +558,16 @@ int kdbGet(KDB *handle, KeySet *ks, Key *parentKey)
 	{
 	case 0: // We don't need an update so let's do nothing
 		keySetName (parentKey, keyName(initialParent));
+		elektraSplitUpdateFileName(split, handle, parentKey);
 		keyDel (initialParent);
 		elektraSplitDel (split);
+		errno = errnosave;
 		return 0;
 	case -1: goto error;
 	// otherwise falltrough
 	}
 
+	// Appoint keys (some in the bypass)
 	if(elektraSplitAppoint (split, handle, ks) == -1)
 	{
 		ELEKTRA_SET_ERROR (38, parentKey, "error in elektraSplitAppoint");
@@ -552,8 +584,8 @@ int kdbGet(KDB *handle, KeySet *ks, Key *parentKey)
 	/* Now postprocess the updated keysets */
 	if (elektraSplitGet (split, parentKey, handle) == -1)
 	{
-		ELEKTRA_SET_ERROR(8, parentKey, keyName(ksCurrent(ks)));
-		goto error;
+		ELEKTRA_ADD_WARNING(108, parentKey, keyName(ksCurrent(ks)));
+		// continue, because sizes are already updated
 	}
 
 	/* We are finished, now just merge everything to returned */
@@ -561,14 +593,18 @@ int kdbGet(KDB *handle, KeySet *ks, Key *parentKey)
 	elektraSplitMerge (split, ks);
 
 	keySetName (parentKey, keyName(initialParent));
+	elektraSplitUpdateFileName(split, handle, parentKey);
 	keyDel (initialParent);
 	elektraSplitDel (split);
+	errno = errnosave;
 	return 1;
 
 error:
 	keySetName (parentKey, keyName(initialParent));
+	elektraSplitUpdateFileName(split, handle, parentKey);
 	keyDel (initialParent);
 	elektraSplitDel (split);
+	errno = errnosave;
 	return -1;
 }
 
@@ -586,9 +622,9 @@ error:
 static int elektraSetPrepare(Split *split, Key *parentKey, Key **errorKey)
 {
 	int any_error = 0;
-	for(size_t p=0; p<COMMIT_PLUGIN; ++p)
+	for(size_t i=0; i<split->size;i++)
 	{
-		for(size_t i=0; i<split->size;i++)
+		for(size_t p=0; p<COMMIT_PLUGIN; ++p)
 		{
 			int ret = 0; // last return value
 
@@ -607,8 +643,22 @@ static int elektraSetPrepare(Split *split, Key *parentKey, Key **errorKey)
 						backend->setplugins[p],
 						split->keysets[i],
 						parentKey);
+
+#if VERBOSE && DEBUG
+				printf ("Prepare %s with keys %d in plugin: %d, split: %d, ret: %d\n",
+						keyName(parentKey), ksGetSize(split->keysets[i]), p, i, ret);
+#endif
+
 				if(p == 0)
 				{
+					if (ret == 0)
+					{
+						// resolver says that sync is
+						// not needed, so we
+						// skip other pre-commit
+						// plugins
+						break;
+					}
 					keySetString (split->parents[i],
 						keyString(parentKey));
 				}
@@ -649,16 +699,29 @@ static void elektraSetCommit(Split *split, Key *parentKey)
 			int ret = 0;
 			Backend *backend = split->handles[i];
 
-			ksRewind (split->keysets[i]);
 			if(backend->setplugins[p])
 			{
-				keySetString(parentKey,
-					keyString(split->parents[i]));
+				if (p!=COMMIT_PLUGIN)
+				{
+					keySetString(parentKey,
+						keyString(split->parents[i]));
+				}
 				keySetName(parentKey, keyName(split->parents[i]));
+#if DEBUG && VERBOSE
+				printf ("elektraSetCommit: %p # %d with %s - %s\n",
+						backend, p, keyName(parentKey), keyString(parentKey));
+#endif
+				ksRewind (split->keysets[i]);
 				ret = backend->setplugins[p]->kdbSet (
 						backend->setplugins[p],
 						split->keysets[i],
 						parentKey);
+				if (p==COMMIT_PLUGIN)
+				{
+					// name of non-temp file
+					keySetString (split->parents[i],
+						keyString(parentKey));
+				}
 			}
 
 			if (ret == -1)
@@ -707,107 +770,65 @@ static void elektraSetRollback(Split *split, Key *parentKey)
 
 
 /** @brief Set keys in an atomic and universal way.
- *
- * With @p parentKey you can only store a part of the given keyset.
- * When other keys also belong to a backend, they will be used too,
- * even when they are above @p parentKey.
- *
- * @par Details:
- *
- * If you pass a parentKey without a name the whole keyset will be set
- * in an atomic way.
- *
- * Each key is checked with keyNeedSync() before being actually committed. So
- * only changed keys are updated. If no key of a backend needs to be synced
- * any affairs to backends omitted and 0 is returned.
- *
- * @par Errors:
- *
- * If some error occurs, kdbSet() will stop. In this situation the KeySet
- * internal cursor will be set on the key that generated the error.
- *
- * None of the keys are actually commited.
- *
- * You should present the error message to the user and let the user decide what
- * to do. Possible solutions are:
- * - repeat the same kdbSet (for temporary errors)
- * - remove the key and set it again (for validation or type errors)
- * - change the value and try it again (for validation errors)
- * - do a kdbGet and then (for conflicts ...)
- *   - set the same keyset again (in favour of what was set by this user)
- *   - drop the old keyset (in favour of what was set elsewhere)
- * - export the configuration into a file (for unresolvable errors)
- *
- * @par Example of how this method can be used:
- * @code
-int i;
-KeySet *myConfig = ksNew(0,KS_END);
-Key *parentKey = keyNew("system/sw/MyApp",KEY_END);
-KDB *handle = kdbOpen(pkey);
-
-kdbGet(handle, ks, parentKey); // kdbGet needs to be called first!
-KeySet *base = ksDup(ks); // save a copy of original keyset
-
-// change the keys within ks
-
-KeySet *ours = ksDup(ks); // save a copy of our keyset
-int ret=kdbSet(handle, ks, parentKey);
-while (ret == -1) // as long as we have an error
-{
-	// We got an error. Warn user.
-	Key *problemKey = ksCurrent(ks);
-	// parentKey has the errorInformation
-	// problemKey is the faulty key (may be null)
-	int userInput = showElektraErrorDialog (parentKey, problemKey);
-	switch (userInput)
-	{
-	case INPUT_USE_OURS:
-		kdbGet(handle, ks, parentKey); // refresh key database
-		ksDel(ks);
-		ks = ours;
-		break;
-	case INPUT_DO_MERGE:
-		kdbGet(handle, ks, parentKey); // refresh key database
-		KeySet * res=doElektraMerge(ours, ks, base);
-		ksDel(ks);
-		ks = res;
-		break;
-	case INPUT_USE_THEIRS:
-		// should always work, we just write what we got
-		// but to be sure always give the user another way
-		// to exit the loop
-		kdbGet(handle, ks, parentKey); // refresh key database
-		break;
-	...
-	}
-	ret=kdbSet(handle, ks, parentKey);
-}
-
-ksDel (ours);
-ksDel (base);
-ksDel (ks); // delete the in-memory configuration
-
-kdbClose(handle, parentKey); // no more affairs with the key database.
-keyDel(parentKey);
-
- * @endcode
- *
- * showElektraErrorDialog() and doElektraMerge() need to be implemented
- * by the user. For doElektraMerge a 3-way merge algorithm exists in
- * libelektra-tools.
  *
  * @pre kdbGet() must be called before kdbSet():
  *    - initially (after kdbOpen())
  *    - after conflict errors in kdbSet().
  *
+ * @pre The @p returned KeySet must be a valid KeySet, e.g. constructed
+ *     with ksNew().
+ *
+ * @pre The @p parentKey Key must be a valid Key, e.g. constructed with
+ *
+ * With @p parentKey you can give an hint which part of the given keyset
+ * is of interest for you. Then you promise, you only modified or
+ * removed keys below this key.
+ *
+ * @par Errors
+ * If some error occurs, kdbSet() will stop. In this situation the KeySet
+ * internal cursor will be set on the key that generated the error.
+ * None of the keys are actually committed in this situation (no
+ * configuration file will be modified).
+ *
+ * In case of errors you should present the error message to the user and let the user decide what
+ * to do. Possible solutions are:
+ * - remove the problematic key and use kdbSet() again (for validation or type errors)
+ * - change the value of the problematic key and use kdbSet() again (for validation errors)
+ * - do a kdbGet() (for conflicts, i.e. error 30) and then
+ *   - set the same keyset again (in favour of what was set by this user)
+ *   - drop the old keyset (in favour of what was set from another application)
+ *   - merge the original, your own and the other keyset
+ * - export the configuration into a file (for unresolvable errors)
+ * - repeat the same kdbSet might be of limited use if the operator does
+ *   not explicitly request it, because temporary
+ *   errors are rare and its unlikely that they fix themselves
+ *   (e.g. disc full, permission problems)
+ *
+ * @par Optimization
+ * Each key is checked with keyNeedSync() before being actually committed. So
+ * only changed keys are updated. If no key of a backend needs to be synced
+ * any affairs to backends are omitted and 0 is returned.
+ *
+ * @snippet kdbset.c set
+ *
+ * showElektraErrorDialog() and doElektraMerge() need to be implemented
+ * by the user of Elektra. For doElektraMerge a 3-way merge algorithm exists in
+ * libelektra-tools.
+ *
  * @param handle contains internal information of @link kdbOpen() opened @endlink key database
  * @param ks a KeySet which should contain changed keys, otherwise nothing is done
- * @param parentKey holds the information below which key keys should be set, see above
+ * @param parentKey is used to add warnings and set an error
+ *         information. Additionally, its name is an hint which keys
+ *         should be committed (it is possible that more are changed).
+ *           - cascading keys (starting with /) will set the path in all namespaces
+ *           - / will commit all keys
+ *           - meta-names will be rejected (error 104)
+ *           - empty/invalid (error 105)
  * @retval 1 on success
- * @retval 0 if nothing had to be done
- * @retval -1 on failure
+ * @retval 0 if nothing had to be done, no changes in KDB
+ * @retval -1 on failure, no changes in KDB
  * @see keyNeedSync()
- * @see ksNext(), ksCurrent() for iteration over the keyset
+ * @see ksCurrent() contains the error key
  * @see kdbOpen() and kdbGet() that must be called first
  * @see kdbClose() that must be called afterwards
  * @ingroup kdb
@@ -822,51 +843,27 @@ int kdbSet(KDB *handle, KeySet *ks, Key *parentKey)
 
 	if (ns == KEY_NS_META)
 	{
-		ELEKTRA_SET_ERROR(104, parentKey,
-				"invalid key name passed to kdbSet");
+		ELEKTRA_SET_ERRORF(104, parentKey,
+				"metakey with name \"%s\" passed to kdbSet",
+				keyName(parentKey));
 		return -1;
 	}
 
+	if (ns == KEY_NS_EMPTY)
+	{
+		ELEKTRA_ADD_WARNING(105, parentKey,
+				"invalid key name passed to kdbSet");
+	}
+
 	if(!handle || !ks)
 	{
 		ELEKTRA_SET_ERROR (37, parentKey, "handle or ks null pointer");
 		return -1;
 	}
 
+	int errnosave = errno;
 	Key *initialParent = keyDup(parentKey);
 
-	if (ns == KEY_NS_CASCADING)
-	{
-		// TODO: does not guarantee atomic commit
-		keySetName(parentKey, "user");
-		keyAddName(parentKey, keyName(initialParent));
-		if (kdbSet(handle, ks, parentKey) == -1)
-		{
-			elektraKeySetName(parentKey,
-					keyName(initialParent),
-					KEY_CASCADING_NAME);
-			keyDel(initialParent);
-			return -1;
-		}
-
-		keySetName(parentKey, "system");
-		keyAddName(parentKey, keyName(initialParent));
-		if (kdbSet(handle, ks, parentKey) == -1)
-		{
-			elektraKeySetName(parentKey,
-					keyName(initialParent),
-					KEY_CASCADING_NAME);
-			keyDel(initialParent);
-			return -1;
-		}
-
-		elektraKeySetName(parentKey,
-				keyName(initialParent),
-				KEY_CASCADING_NAME);
-		keyDel(initialParent);
-		return 1;
-	}
-
 #if DEBUG && VERBOSE
 	fprintf(stderr, "now in new kdbSet (%s)\n", keyName(parentKey));
 #endif
@@ -880,20 +877,37 @@ int kdbSet(KDB *handle, KeySet *ks, Key *parentKey)
 		goto error;
 	}
 
-	if(elektraSplitDivide(split, handle, ks) == -1)
+	// 1.) Search for syncbits
+	int syncstate = elektraSplitDivide(split, handle, ks);
+	if(syncstate == -1)
 	{
 		ELEKTRA_SET_ERROR(8, parentKey, keyName(ksCurrent(ks)));
 		goto error;
 	}
+	ELEKTRA_ASSERT(syncstate == 0 || syncstate == 1);
 
-	if(elektraSplitSync(split) == 0)
+	// 2.) Search for changed sizes
+	syncstate |= elektraSplitSync(split);
+	ELEKTRA_ASSERT(syncstate == 0 || syncstate == 1);
+	if (syncstate != 1)
 	{
 		/* No update is needed */
 		keySetName (parentKey, keyName(initialParent));
+		if(syncstate == -1)
+		{
+			ELEKTRA_SET_ERROR(107, parentKey, "Assert failed: invalid namespace");
+		}
+		else if(syncstate < -1)
+		{
+			ELEKTRA_SET_ERROR(107, parentKey,
+					 keyName(split->parents[-syncstate-2]));
+		}
 		keyDel (initialParent);
 		elektraSplitDel (split);
-		return 0;
+		errno = errnosave;
+		return syncstate == 0 ? 0 : -1;
 	}
+	ELEKTRA_ASSERT(syncstate == 1);
 
 	elektraSplitPrepare(split);
 
@@ -906,11 +920,17 @@ int kdbSet(KDB *handle, KeySet *ks, Key *parentKey)
 
 	elektraSplitUpdateSize(split);
 
+	for (size_t i=0; i<ks->size; ++i)
+	{
+		// remove all flags from all keys
+		clear_bit(ks->array[i]->flags, KEY_FLAG_SYNC);
+	}
+
 	keySetName(parentKey, keyName(initialParent));
 	keyDel(initialParent);
 	elektraSplitDel(split);
-	for (size_t i=0; i<ks->size; ++i) ks->array[i]->flags = 0;
 
+	errno = errnosave;
 	return 1;
 
 error:
@@ -929,6 +949,7 @@ error:
 	keySetName(parentKey, keyName(initialParent));
 	keyDel(initialParent);
 	elektraSplitDel(split);
+	errno = errnosave;
 	return -1;
 }
 
