@@ -14,6 +14,18 @@
  ***************************************************************************/
 
 
+/** @class doxygenNamespaces
+ *
+ * @brief .
+ *
+ * - @p spec/something for specification of other keys
+ * - @p proc/something for in-memory keys (e.g. command-line)
+ * - @p dir/something for dir keys in current working directory
+ * - @p system/something for system keys in /etc or /
+ * - @p user/something for user keys in home directory
+ * - @p user:username/something for other users (deprecated: kdbGet() + kdbSet() currently unsupported)
+ * - @p /something for cascading keys (actually refers to one of the above, see also ksLookup())
+ */
 
 
 /**
@@ -38,12 +50,13 @@
  *   separator.
  * - A *key base name* (see keySetBaseName() and keyAddBaseName()) is
  *   the last part of the key name.
- * - A namespace denotes the place the key comes from:
- *   - _user_ keys come from user's home directories
- *   - _system_ keys come from systems etc directories
  * - A *C-String* is a null terminated sequence of characters.
  *   So \\0 (null-character) must not occur within a C-String.
  *
+ * @par Namespaces
+ * A namespace denotes the place the key comes from:
+ * @copydetails doxygenNamespaces
+ *
  *
  * @note The rules are currently not formally specified and are subject
  * of change in the next major release.
@@ -101,11 +114,19 @@
  * - \\ (backslash) is the escape character for the situations as
  *   described here (and only these).
  *   The \\ character must only be escaped, when one of the following
- *   rules apply. So there is no stray escape character possible.
- * - \\/ allows to escape /
- * - \\\\/ allows to use \\ as character before / (and so on)
+ *   rules apply.
+ * - Stray escape characters are only possible in the end of the string.
+ * - \\/ allows to escape / (any uneven number of \\).
+ *   Does not introduce a new part.
+ * - Any uneven number N of \\ before / allows you to escape / with the
+ *   N/2 of \\ prefixed.
+ *   Does not introduce a new part.
+ * - \\\\/ allows to use \\ as character before / and introduces a new
+ *   part.
+ * - Any even number N of \\ before / allows you to have N/2 of \\
+ *   prefixed before a / which introduces a new part.
  * - Use \\. and \\.. if you want your key name part to represent . and ..
- * - \\\\. and \\\\.. allows to use \\ as character before . and .. (and so on)
+ * - \\\\. and \\\\.. allows us to use \\ as character before . and .. (and so on)
  * - Use \\% if you want your key name part to start with \% (and does
  *   not represent an empty name)
  * - Use \\\\% allows to use \\ as character before \% (and so on)
@@ -140,6 +161,7 @@
  *   Additionally, it is possible to start the old version of the app,
  *   using @p /sw/myapp/#2.
  *
+ * @{
  */
 
 
@@ -196,6 +218,9 @@ keyName(key); // you would expect "" here
 keyDel(key);
  * @endcode
  *
+ * Valid key names are:
+ * @copydetails doxygenNamespaces
+ *
  * @note Note that the Key structure keeps its own size field that is calculated
  * by library internal calls, so to avoid inconsistencies, you
  * must never use the pointer returned by keyName() method to set a new
@@ -222,18 +247,6 @@ const char *keyName(const Key *key)
 	return key->key;
 }
 
-const void *keyUnescapedName(const Key *key)
-{
-	if (!key) return 0;
-
-	if (!key->key) {
-		return "";
-	}
-
-	return key->key+key->keySize;
-}
-
-
 /**
  * Bytes needed to store the key name without owner.
  *
@@ -260,18 +273,6 @@ ssize_t keyGetNameSize(const Key *key)
 }
 
 
-ssize_t keyGetUnescapedNameSize(const Key *key)
-{
-	if (!key) return -1;
-
-	if (!key->key)
-	{
-		return 0;
-	}
-	else return key->keyUSize;
-}
-
-
 
 
 /**
@@ -364,14 +365,66 @@ ssize_t elektraFinalizeEmptyName(Key *key)
 	return key->keySize;
 }
 
+static void elektraHandleUserName(Key *key, const char* newName)
+{
+	const size_t userLength=sizeof("user");
+	key->keyUSize=key->keySize=userLength;
+
+	const char delim = newName[userLength-1];
+	// no owner, we are finished
+	if (delim == '/' || delim == '\0') return;
+	ELEKTRA_ASSERT(delim == ':');
+
+	// handle owner (compatibility, to be removed)
+	keyNameGetOneLevel(newName, &key->keyUSize);
+	const size_t ownerLength=key->keyUSize-userLength;
+	++key->keyUSize;
+	char *owner=elektraMalloc(ownerLength+1);
+	if (!owner) return; // out of memory, ok for owner
+	strncpy(owner,newName+userLength,ownerLength);
+	owner[ownerLength]=0;
+	keySetOwner(key, owner);
+	elektraFree (owner);
+}
+
+static void elektraRemoveKeyName(Key *key)
+{
+	if (key->key) elektraFree(key->key);
+	key->key=0;
+	key->keySize=0;
+	key->keyUSize=0;
+}
+
+/**
+ * @brief Checks if in name is something else other than slashes
+ *
+ * @param name the name to check
+ *
+ * @retval 0 if not only slashes
+ * @retval 1 if only slashes
+ */
+static int elektraOnlySlashes(const char *name)
+{
+	size_t nameLen = strlen(name);
+	for (size_t i=0; i<nameLen; ++i)
+	{
+		if (name[i] != '/') return 0; // not only slashes
+	}
+	return 1; // only slashes
+}
+
+
 
 /**
  * Set a new name to a key.
  *
- * A valid name is of the forms:
- * - @p system/something
- * - @p user/something
- * - @p user:username/something
+ * A valid name is one of the forms:
+ * @copydetails doxygenNamespaces
+ *
+ * An invalid name either has an invalid namespace or
+ * a wrongly escaped \\ at the end of the name.
+ *
+ * See @link keyname key names @endlink for the exact rules.
  *
  * The last form has explicitly set the owner, to let the library
  * know in which user folder to save the key. A owner is a user name.
@@ -407,158 +460,57 @@ ssize_t keySetName(Key *key, const char *newName)
 ssize_t elektraKeySetName(Key *key, const char *newName,
 		option_t options)
 {
-	size_t length;
-	size_t rootLength, userLength, systemLength, ownerLength;
-	char *p=0;
-
 	if (!key) return -1;
 	if (test_bit(key->flags,  KEY_FLAG_RO_NAME)) return -1;
 
-	if (key->key) elektraFree(key->key);
-	key->key = 0;
-	key->keySize=1; /* equal to length plus room for \\0 */
+	elektraRemoveKeyName(key);
+	if (!(options & KEY_META_NAME)) keySetOwner (key, NULL);
 
-	/* handle null new key name, removing the old name */
-	if (!newName || !(length=elektraStrLen(newName)-1))
+	switch(keyGetNameNamespace(newName))
 	{
+	case KEY_NS_NONE: ELEKTRA_ASSERT(0);
+	case KEY_NS_EMPTY:
 		elektraFinalizeEmptyName(key);
-		return 0; // we need to return 0 because of specification
+		return 0; // as documented
+	case KEY_NS_CASCADING: key->keyUSize=1;key->keySize=sizeof("/"); break;
+	case KEY_NS_SPEC: key->keyUSize=key->keySize=sizeof("spec"); break;
+	case KEY_NS_PROC: key->keyUSize=key->keySize=sizeof("proc"); break;
+	case KEY_NS_DIR: key->keyUSize=key->keySize=sizeof("dir"); break;
+	case KEY_NS_USER: elektraHandleUserName(key, newName); break;
+	case KEY_NS_SYSTEM: key->keyUSize=key->keySize=sizeof("system"); break;
+	case KEY_NS_META:
+		if (!(options & KEY_META_NAME)) return -1;
+		keyNameGetOneLevel(newName,&key->keySize);
+		key->keyUSize = ++ key->keySize; // for null
+		break;
+	} // Note that we abused keyUSize for cascading and user:owner
+
+	const size_t length = elektraStrLen(newName);
+	key->key=elektraMalloc(key->keySize*2);
+	memcpy(key->key, newName, key->keySize);
+	if (length == key->keyUSize || length == key->keySize)
+	{	// use || because full length is keyUSize in user, but keySize for /
+		// newName consisted of root only
+		elektraFinalizeName(key);
+		return key->keyUSize;
 	}
 
-	rootLength=keyNameGetFullRootNameSize(newName)-1;
-	if (!(options & KEY_CASCADING_NAME) && !rootLength)
+	if (elektraOnlySlashes(newName+key->keyUSize-1))
 	{
-		return -1;
-	}
-	userLength=sizeof("user")-1;
-	systemLength=sizeof("system")-1;
-	ownerLength=rootLength-userLength;
-	
-	if (ownerLength>0) --ownerLength;
-
-	if ((options & KEY_EMPTY_NAME) &&
-		(!strcmp(newName, "")))
-	{
-		return elektraFinalizeEmptyName(key);
-	}
-	if ((options & KEY_CASCADING_NAME) &&
-		(newName[0] == '/'))
-	{
-		if (!strcmp(newName, "/"))
-		{
-			key->key = elektraCalloc(4);
-			key->key[0] = '/';
-			key->keySize=2;
-			elektraFinalizeName(key);
-			return key->keySize;
-		}
-		/* handle cascading key names */
-		rootLength = 1;
-	}
-	else if (options & KEY_META_NAME)
-	{
-		size_t size = 0;
-		p=keyNameGetOneLevel(newName,&size);
-		rootLength = size+1;
-	}
-	else if (keyNameIsUser(newName))
-	{
-		/* handle "user*" */
-		if (length > userLength)
-		{
-			/* handle "user?*" */
-			if (*(newName+userLength)==':')
-			{
-				/* handle "user:*" */
-				if (ownerLength > 0)
-				{
-					char *owner;
-					p=elektraMalloc(ownerLength+1);
-					if (NULL==p) goto error_mem;
-					owner=p;
-					strncpy(owner,newName+userLength+1,ownerLength);
-					owner[ownerLength]=0;
-					keySetOwner(key, owner);
-					elektraFree (owner);
-				}
-				key->keySize+=length-ownerLength-1;  /* -1 is for the ':' */
-			} else if (*(newName+userLength)!=KDB_PATH_SEPARATOR) {
-				/* handle when != "user/ *" */
-				return -1;
-			} else {
-				/* handle regular "user/ *" */
-				key->keySize+=length;
-			}
-		} else {
-			/* handle "user" */
-			key->keySize+=userLength;
-		}
-
-		rootLength  = userLength+1;
-	}
-	else if (keyNameIsSystem(newName))
-	{
-		/* handle "system*" */
-		if (length > systemLength && *(newName+systemLength)!=KDB_PATH_SEPARATOR)
-		{	/* handle when != "system/ *" */
-			return -1;
-		}
-		key->keySize+=length;
-
-		keySetOwner (key, NULL);
-
-		rootLength  = systemLength+1;
-	}
-	else
-	{
-		/**Unsupported key name */
-		return -1;
+		elektraFinalizeName(key);
+		return key->keySize;
 	}
 
-	/*
-	   At this point:
-	   - key->key has no memory (re)allocated yet
-	   - key->keySize has number of bytes that will be allocated for key name
-	     with already removed owner. (even though we do not need it)
-	   - owner is already set
-	   - rootLength is sizeof("user") or sizeof("system")
-	*/
-
-	/* Allocate memory for key->key */
-	p=elektraCalloc(rootLength);
-	if (NULL==p) goto error_mem;
-	key->key=p;
-
-	/* copy the root of newName to final destination */
-	strncpy(key->key,newName,rootLength);
-
-	/* finish root name for keyAddName() */
-	key->keySize=rootLength;
-	key->key[rootLength-1] = '\0';
-
-	size_t size = 0;
-	if ((options & KEY_CASCADING_NAME) &&
-		(newName[0] == '/'))
-	{
-		p = (char*)newName;
-		size=1;
-	}
-	else
-	{
-		/* skip namespace we already processed */
-		p=keyNameGetOneLevel(newName,&size);
-	}
-
-	return keyAddName(key, p+size);
-
-error_mem:
-	return -1;
+	key->key[key->keySize-1] = '\0';
+	const ssize_t ret = keyAddName(key, newName+key->keyUSize);
+	if (ret == -1) elektraRemoveKeyName(key);
+	else return key->keySize;
+	return ret;
 }
 
 
 
 
-
 /**
  * Bytes needed to store the key name including user domain and ending NULL.
  *
@@ -653,41 +605,23 @@ ssize_t keyGetFullName(const Key *key, char *returnedName, size_t maxSize)
 
 
 /**
- * For currently valid namespaces see #elektraNamespace.
- *
- * @version 0.8.10
- * Added method to kdbproposal.h
- *
- * @note This method might be enhanced. You do not have any guarantee
- * that, when for a specific name #KEY_NS_META
- * is returned today, that it still will be returned after the next
- * recompilation. So make sure that your compiler gives you a warning
- * for unhandled switches (gcc: -Wswitch or -Wswitch-enum if you
- * want to handle default) and look out for those warnings.
- *
- * @param key the key object to work with
- * @return the namespace of a key.
- * @ingroup keyname
- *
+ * @internal
  */
-elektraNamespace keyGetNamespace(const Key *key)
+elektraNamespace keyGetNameNamespace(const char *name)
 {
-	if (!key) return KEY_NS_NONE;
-
-	if (!key->key) return KEY_NS_EMPTY;
-	if (!strcmp(key->key, "")) return KEY_NS_EMPTY;
-
-	if (key->key[0] == '/') return KEY_NS_CASCADING;
-
-	if (keyIsUser (key)) return KEY_NS_USER;
-	if (keyIsSystem (key)) return KEY_NS_SYSTEM;
-
+	if (!name) return KEY_NS_EMPTY;
+	if (!strcmp(name, "")) return KEY_NS_EMPTY;
+	if (name[0] == '/') return KEY_NS_CASCADING;
+	else if (keyNameIsSpec(name)) return KEY_NS_SPEC;
+	else if (keyNameIsProc(name)) return KEY_NS_PROC;
+	else if (keyNameIsDir(name)) return KEY_NS_DIR;
+	else if (keyNameIsUser(name)) return KEY_NS_USER;
+	else if (keyNameIsSystem(name)) return KEY_NS_SYSTEM;
 	return KEY_NS_META;
 }
 
 
 
-
 /**
  * @brief Returns a pointer to the internal unescaped key name where the @p basename starts.
  *
@@ -706,10 +640,13 @@ elektraNamespace keyGetNamespace(const Key *key)
  * method to change the name, but you should use keySetBaseName()
  * instead.
  *
+ * @note Do not assume that keyBaseName() points to the same region as
+ * keyName() does.
+ *
  * @param key the object to obtain the basename from
  * @return a pointer to the basename
- * @return "" when the key has no (base)name
- * @return 0 on NULL pointer
+ * @retval "" when the key has no (base)name
+ * @retval 0 on NULL pointer
  * @see keyGetBaseName(), keyGetBaseNameSize()
  * @see keyName() to get a pointer to the name
  * @see keyOwner() to get a pointer to the owner
@@ -854,11 +791,18 @@ ssize_t keyAddBaseName(Key *key, const char *baseName)
 	if (test_bit(key->flags,  KEY_FLAG_RO_NAME)) return -1;
 	if (!key->key) return -1;
 
-	size_t size=0;
 	char *escaped = elektraMalloc (strlen (baseName) * 2 + 2);
 	elektraEscapeKeyNamePart(baseName, escaped);
-	size = strlen (escaped);
-	key->keySize += size + 1;
+	size_t len = strlen (escaped);
+	if (!strcmp(key->key, "/"))
+	{
+		key->keySize += len;
+	}
+	else
+	{
+		key->keySize += len + 1;
+	}
+
 	elektraRealloc ((void**)&key->key, key->keySize*2);
 	if (!key->key)
 	{
@@ -866,8 +810,11 @@ ssize_t keyAddBaseName(Key *key, const char *baseName)
 		return -1;
 	}
 
-	key->key[key->keySize - size - 2] = KDB_PATH_SEPARATOR;
-	memcpy (key->key + key->keySize - size - 1, escaped, size);
+	if (strcmp(key->key, "/"))
+	{
+		key->key[key->keySize - len - 2] = KDB_PATH_SEPARATOR;
+	}
+	memcpy (key->key + key->keySize - len - 1, escaped, len);
 
 	elektraFree (escaped);
 
@@ -876,71 +823,140 @@ ssize_t keyAddBaseName(Key *key, const char *baseName)
 	return key->keySize;
 }
 
+/**
+ * @internal
+ *
+ * @brief Used by keyAddName
+ *
+ * Will remove one level of key, even if key->key is not null terminated
+ * also handles cascading keys and sets avoidSlash properly.
+ *
+ * @param key to remove one level
+ * @param [out] avoidSlash set to 1 if / is already present (cascading)
+ */
+static void elektraRemoveOneLevel(Key *key, int *avoidSlash)
+{
+	int levels = 0;
+	char *x = key->key;
+	size_t xsize = 0;
+	size_t sizeOfLastLevel = 0;
+	char * const last = &key->key[key->keySize];
+	const char save = *last;
+	*last = 0;
+
+	while (*(x=keyNameGetOneLevel(x+xsize,&xsize)))
+	{
+		sizeOfLastLevel = xsize;
+		levels++;
+	}
+
+	if (levels > 1)
+	{
+		key->keySize -= sizeOfLastLevel+1;
+		key->key[key->keySize]=0;
+	}
+	else if (*key->key == '/') // cascading key
+	{
+		// strip down to root
+		key->keySize = 1;
+		*avoidSlash = 1;
+	}
+	*last = save;
+}
+
 /**
  * @brief Add a already escaped name to the keyname.
  *
- * The same way as in keySetName() this method finds the canonical pathname.
- * Unlike, keySetName() it adds it to an already existing name.
+ * The same way as in keySetName() this method finds the canonical pathname:
+ * - it will ignore /./
+ * - it will remove a level when /../ is used
+ * - it will remove multiple slashes ////
+ *
+ * For example:
+ * @snippet keyName.c add name
+ *
+ * Unlike keySetName() it adds relative to the previous name and
+ * cannot change the namespace of a key.
+ * For example:
+ * @snippet keyName.c namespace
+ *
+ * The passed name needs to be valid according the @link keyname key name rules @endlink.
+ * It is not allowed to:
+ * - be empty
+ * - end with unequal number of \\
  *
  * @param key the key where a name should be added
  * @param newName the new name to append
  *
+ * @since 0.8.11
+ *
+ * @retval size of the new key
  * @retval -1 if key is a null pointer or did not have a valid name before
- * @retval -1 if newName is a null pointer or not a valid name (contains \\ in beginning)
+ * @retval -1 if newName is not a valid escaped name
  * @retval -1 on allocation errors
  * @retval -1 if key was inserted to a keyset before
- * @retval size of the new key
+ * @retval 0 if nothing was done because newName had only slashes, is too short, is empty or is null
+ * @ingroup keyname
  */
 ssize_t keyAddName(Key *key, const char *newName)
 {
 	if (!key) return -1;
-	if (!newName) return key->keySize;
 	if (test_bit(key->flags,  KEY_FLAG_RO_NAME)) return -1;
-	// if (!elektraValidateKeyNamePart(newName)) return -1;
 	if (!key->key) return -1;
+	if (!strcmp(key->key, "")) return -1;
+	if (!newName) return 0;
+	size_t const nameSize = elektraStrLen(newName);
+	if (nameSize < 2) return 0;
+	if (!elektraValidateKeyName(newName, nameSize)) return -1;
 
-	size_t const newSize = key->keySize + elektraStrLen(newName);
-	size_t const rootLength = key->keySize;
+	const size_t origSize = key->keySize;
+	const size_t newSize = origSize + nameSize;
 	elektraRealloc ((void**)&key->key, newSize*2);
 	if (!key->key) return -1;
 
 	size_t size=0;
 	const char * p = newName;
+	int avoidSlash = 0;
 
-	-- key->keySize; // fix keySize for loop below
+	if (*key->key == '/') avoidSlash = key->keySize == 2;
+
+	-- key->keySize; // loop assumes that key->key[key->keySize] is last character and not NULL
 
 	/* iterate over each single folder name removing repeated '/', .  and .. */
-	while (*(p=keyNameGetOneLevel(p+size,&size))) {
-		// printf ("level: %s, size: %d\n", p, size);
+	while (*(p=keyNameGetOneLevel(p+size,&size)))
+	{
 		if (size == 1 && strncmp (p, ".",1) == 0)
 		{
-			/* printf ("ignore .\n"); */
 			continue; /* just ignore current directory */
 		}
-		else if (size == 2 && strncmp (p, "..",2) == 0) /* give away directory */
+		else if (size == 2 && strncmp (p, "..", 2) == 0) /* give away one level*/
 		{
-			key->key[key->keySize] = 0; /* initialize first (valgrind) */
-			while (key->keySize >= rootLength && key->key[key->keySize] != KDB_PATH_SEPARATOR) key->keySize--;
-			/* printf ("do .. (key->keySize: %d), key->key: %s, rootLength: %d, key->keySize: %d\n",
-					key->keySize, key->key, rootLength, key->keySize); */
+			elektraRemoveOneLevel(key, &avoidSlash);
 			continue;
 		}
-		/* Add a '/' to the end of key name */
-		key->key[key->keySize]=KDB_PATH_SEPARATOR;
-		key->keySize++;
-		
+
+		if (!avoidSlash)
+		{
+			/* Add a '/' to the end of key name */
+			key->key[key->keySize]=KDB_PATH_SEPARATOR;
+			key->keySize++;
+		}
+		else
+		{
+			avoidSlash = 0;
+		}
+
 		/* carefully append basenames */
-		memcpy(key->key+key->keySize,p,size);
+		char *d = key->key+key->keySize;
+		memcpy(d,p,size);
 		key->keySize+=size;
 	}
 
-	/* remove unescaped trailing slashes */
-	while (key->key[key->keySize-1] == KDB_PATH_SEPARATOR && key->key[key->keySize-2] != '\\') key->keySize--;
-	key->keySize ++; /*for \\0 ending*/
+	++ key->keySize; /*for \\0 ending*/
 
 	elektraFinalizeName(key);
 
-	return key->keySize;
+	return origSize == key->keySize ? 0 : key->keySize;
 }
 
 
@@ -967,11 +983,13 @@ ssize_t keyAddName(Key *key, const char *newName)
  * A simple example is:
  * @snippet basename.c set base basic
  *
- * If you do not want escaping, use keySetBaseName() instead. E.g. if
- * you want to add an inactive key, use:
+ * If you want to add and not change the basename, use keyAddBaseName()
+ * instead. If you do not want escaping, use keyAddName() instead.
+ *
+ * To add an inactive key name, use:
  * @snippet testabi_key.c base1
  *
- * or when you want to add an array item, use:
+ * When you want to add an array item, use:
  * @snippet testabi_key.c base2
  *
  * @see keyname for more details on special names
@@ -1012,7 +1030,7 @@ ssize_t keySetBaseName(Key *key, const char *baseName)
 
 	if (!baseName)
 	{
-		// just remove base name
+		// just remove base name, so we are finished
 		elektraFinalizeName(key);
 		return key->keySize;
 	}
@@ -1041,189 +1059,7 @@ ssize_t keySetBaseName(Key *key, const char *baseName)
 }
 
 
-
-/*****************************************************
- *         General owner manipulation methods        *
- *****************************************************/
-
-
-
-
 /**
- * Return a pointer to the real internal @p key owner.
- *
- * This is a much more efficient version of keyGetOwner() and you
- * should use it if you are responsible enough to not mess up things.
- * You are not allowed to modify the returned string in any way.
- * If you need a copy of the string, consider to use keyGetOwner() instead.
- *
- * keyOwner() returns "" when there is no keyOwner. The reason is
- * @code
-key=keyNew(0);
-keySetOwner(key,"");
-keyOwner(key); // you would expect "" here
-keySetOwner(key,"system");
-keyOwner(key); // you would expect "" here
- * @endcode
- *
- * @note Note that the Key structure keeps its own size field that is calculated
- * by library internal calls, so to avoid inconsistencies, you
- * must never use the pointer returned by keyOwner() method to set a new
- * value. Use keySetOwner() instead.
- *
- * @param key the key object to work with
- * @return a pointer to internal owner
- * @retval "" when there is no (a empty) owner
- * @retval 0 iff key is a NULL pointer
- * @see keyGetOwnerSize() for the size of the string with concluding 0
- * @see keyGetOwner(), keySetOwner()
- * @see keyName() for name without owner
- * @see keyGetFullName() for name with owner
- * @ingroup keyname
+ * @}
  */
-const char *keyOwner(const Key *key)
-{
-	const char *owner;
 
-	if (!key) return 0;
-	owner = keyValue(keyGetMeta(key, "owner"));
-
-	if (!owner)
-	{
-		return "";
-	}
-
-	return owner;
-}
-
-
-
-
-
-
-/**
- * Return the size of the owner of the Key with concluding 0.
- *
- * The returned number can be used to allocate a string.
- * 1 will returned on an empty owner to store the concluding 0
- * on using keyGetOwner().
- *
- * @code
-char * buffer;
-buffer = malloc (keyGetOwnerSize (key));
-// use buffer and keyGetOwnerSize (key) for maxSize
- * @endcode
- *
- * @note that -1 might be returned on null pointer, so when you
- * directly allocate afterwards its best to check if you will pass
- * a null pointer before.
- *
- * @param key the key object to work with
- * @return number of bytes
- * @return 1 if there is no owner
- * @return -1 on NULL pointer
- * @see keyGetOwner()
- * @ingroup keyname
- */
-ssize_t keyGetOwnerSize(const Key *key)
-{
-	ssize_t size;
-	if (!key) return -1;
-
-	size = keyGetValueSize(keyGetMeta (key, "owner"));
-
-	if (!size || size == -1)
-	{
-		/*errno=KDB_ERR_NODESC;*/
-		return 1;
-	}
-
-	return size;
-}
-
-
-
-/**
- * Return the owner of the key.
- * - Given @p user:someuser/..... return @p someuser
- * - Given @p user:some.user/.... return @p some.user
- * - Given @p user/.... return the current user
- *
- * Only @p user/... keys have a owner.
- * For @p system/... keys (that doesn't have a key owner) an empty
- * string ("") is returned.
- *
- * Although usually the same, the owner of a key is not related to its
- * UID. Owner are related to WHERE the key is stored on disk, while
- * UIDs are related to mode controls of a key.
- *
- * @param key the object to work with
- * @param returnedOwner a pre-allocated space to store the owner
- * @param maxSize maximum number of bytes that fit returned
- * @return number of bytes written to buffer
- * @return 1 if there is no owner
- * @return -1 on NULL pointers
- * @return -1 when maxSize is 0, larger than SSIZE_MAX or too small for ownername
- * @see keySetName(), keySetOwner(), keyOwner(), keyGetFullName()
- * @ingroup keyname
- */
-ssize_t keyGetOwner(const Key *key, char *returnedOwner, size_t maxSize)
-{
-	const char *owner;
-	size_t ownerSize;
-	if (!key) return -1;
-
-	if (!maxSize) return -1;
-	if (!returnedOwner) return -1;
-	if (maxSize > SSIZE_MAX) return -1;
-
-	owner = keyValue(keyGetMeta(key, "owner"));
-	ownerSize = keyGetValueSize(keyGetMeta(key, "owner"));
-
-	if (!owner)
-	{
-		/*errno=KDB_ERR_NODESC;*/
-		returnedOwner[0]=0;
-		return 1;
-	}
-
-	strncpy(returnedOwner,owner,maxSize);
-	if (maxSize < ownerSize) {
-		/*errno=KDB_ERR_TRUNC;*/
-		return -1;
-	}
-	return ownerSize;
-}
-
-
-
-/**
- * Set the owner of a key.
- *
- * A owner is a name of a system user related to a UID.
- * The owner decides on which location on the disc the key
- * goes.
- *
- * A private copy is stored, so the passed parameter can be freed after
- * the call.
- *
- * @param key the key object to work with
- * @param newOwner the string which describes the owner of the key
- * @return the number of bytes actually saved including final NULL
- * @return 1 when owner is freed (by setting 0 or "")
- * @return -1 on null pointer or memory problems
- * @see keySetName(), keyGetOwner(), keyGetFullName()
- * @ingroup keyname
- */
-ssize_t keySetOwner(Key *key, const char *newOwner)
-{
-	if (!key) return -1;
-	if (!newOwner || *newOwner==0)
-	{
-		keySetMeta (key, "owner", 0);
-		return 1;
-	}
-
-	keySetMeta (key, "owner", newOwner);
-	return keyGetOwnerSize (key);
-}
