@@ -380,44 +380,6 @@ ssize_t keySetString(Key *key, const char *newStringValue)
 }
 
 
-/**
- * @brief Set a formatted string
- *
- * @param key the key to set the string value
- * @param format NULL-terminated text format string
- * @param ... more arguments
- *
- * @return the size of the string as set (with including 0)
- */
-ssize_t keySetStringF(Key *key, const char *format, ...)
-{
-	va_list arg_list;
-
-	keySetMeta (key, "binary", 0);
-
-	va_start(arg_list, format);
-	char *p = elektraFormat(format, arg_list);
-	va_end(arg_list);
-
-	if (!p)
-	{
-		return -1;
-	}
-
-	if (key->data.c)
-	{
-		elektraFree(key->data.c);
-	}
-
-	key->data.c = p;
-	key->dataSize = elektraStrLen(key->data.c);
-	set_bit(key->flags, KEY_FLAG_SYNC);
-
-	return key->dataSize;
-}
-
-
-
 
 /**
  * Get the value of a key as a binary.
@@ -592,191 +554,3 @@ ssize_t keySetRaw(Key *key, const void *newBinary, size_t dataSize)
 	return keyGetValueSize (key);
 }
 
-
-
-
-
-/********************************************* 
- *    General comment manipulation methods   *
- *********************************************/
-
-
-
-
-/**
- * Return a pointer to the real internal @p key comment.
- *
- * This is a much more efficient version of keyGetComment() and you
- * should use it if you are responsible enough to not mess up things.
- * You are not allowed to change anything in the memory region the
- * returned pointer points to.
- *
- * keyComment() returns "" when there is no keyComment. The reason is
- * @code
-key=keyNew(0);
-keySetComment(key,"");
-keyComment(key); // you would expect "" here
-keyDel(key);
- * @endcode
- *
- * See keySetComment() for more information on comments.
- *
- * @note Note that the Key structure keeps its own size field that is calculated
- * by library internal calls, so to avoid inconsistencies, you
- * must never use the pointer returned by keyComment() method to set a new
- * value. Use keySetComment() instead.
- *
- * @param key the key object to work with
- * @return a pointer to the internal managed comment
- * @return "" when there is no comment
- * @return 0 on NULL pointer
- * @see keyGetCommentSize() for size and keyGetComment() as alternative
- * @ingroup keyvalue
- */
-const char *keyComment(const Key *key)
-{
-	const char *comment;
-
-	if (!key) return 0;
-	comment = keyValue(keyGetMeta(key, "comment"));
-
-	if (!comment)
-	{
-		/*errno=KDB_ERR_NOKEY;*/
-		return "";
-	}
-
-	return comment;
-}
-
-
-
-
-/**
- * Calculates number of bytes needed to store a key comment, including
- * final NULL.
- *
- * Use this method to know to size for allocated memory to retrieve
- * a key comment.
- *
- * See keySetComment() for more information on comments.
- *
- * For an empty key name you need one byte to store the ending NULL.
- * For that reason 1 is returned.
- *
- * @code
-char *buffer;
-buffer = malloc (keyGetCommentSize (key));
-// use this buffer to store the comment
-// pass keyGetCommentSize (key) for maxSize
- * @endcode
- *
- * @param key the key object to work with
- * @return number of bytes needed
- * @return 1 if there is no comment
- * @return -1 on NULL pointer
- * @see keyGetComment(), keySetComment()
- * @ingroup keyvalue
- */
-ssize_t keyGetCommentSize(const Key *key)
-{
-	ssize_t size;
-	if (!key) return -1;
-
-	size = keyGetValueSize(keyGetMeta(key, "comment"));
-
-	if (!size || size == -1)
-	{
-		/*errno=KDB_ERR_NODESC;*/
-		return 1;
-	}
-
-	return size;
-}
-
-
-
-/**
- * Get the key comment.
- *
- * @section comment Comments
- *
- * A Key comment is description for humans what this key is for. It may be a
- * textual explanation of valid values, when and why a user or administrator
- * changed the key or any other text that helps the user or administrator related
- * to that key.
- *
- * Don't depend on a comment in your program. A user is
- * always allowed to remove or change it in any way he wants to. But you are
- * allowed or even encouraged to always show the content of the comment
- * to the user and allow him to change it.
- *
- * @param key the key object to work with
- * @param returnedComment pre-allocated memory to copy the comments to
- * @param maxSize number of bytes that will fit returnedComment
- * @return the number of bytes actually copied to @p returnedString, including
- * 	final NULL
- * @return 1 if the string is empty
- * @return -1 on NULL pointer
- * @return -1 if maxSize is 0, not enough to store the comment or when larger then SSIZE_MAX
- * @see keyGetCommentSize(), keySetComment()
- * @ingroup keyvalue
- */
-ssize_t keyGetComment(const Key *key, char *returnedComment, size_t maxSize)
-{
-	const char *comment;
-	size_t commentSize;
-	if (!key) return -1;
-
-	if (!maxSize) return -1;
-	if (!returnedComment) return -1;
-	if (maxSize > SSIZE_MAX) return -1;
-
-	comment = keyValue(keyGetMeta(key, "comment"));
-	commentSize = keyGetValueSize(keyGetMeta(key, "comment"));
-
-	if (!comment)
-	{
-		/*errno=KDB_ERR_NODESC;*/
-		returnedComment[0]=0;
-		return 1;
-	}
-
-	strncpy(returnedComment,comment,maxSize);
-	if (maxSize < commentSize) {
-		/*errno=KDB_ERR_TRUNC;*/
-		return -1;
-	}
-	return commentSize;
-}
-
-
-
-
-
-/**
- * Set a comment for a key.
- *
- * A key comment is like a configuration file comment.
- * See keySetComment() for more information.
- *
- * @param key the key object to work with
- * @param newComment the comment, that can be freed after this call.
- * @return the number of bytes actually saved including final NULL
- * @return 0 when the comment was freed (newComment NULL or empty string)
- * @return -1 on NULL pointer or memory problems
- * @see keyGetComment()
- * @ingroup keyvalue
- */
-ssize_t keySetComment(Key *key, const char *newComment)
-{
-	if (!key) return -1;
-	if (!newComment || *newComment==0)
-	{
-		keySetMeta (key, "comment", 0);
-		return 1;
-	}
-
-	keySetMeta (key, "comment", newComment);
-	return keyGetCommentSize (key);
-}
