@@ -52,7 +52,7 @@
  * Splits up a keyset into multiple keysets where each
  * of them will passed to the correct kdbSet().
  *
- * Initially the size is APPROXIMATE_NR_OF_BACKENDS.
+ * Initially the size is 0 and alloc is APPROXIMATE_NR_OF_BACKENDS.
  *
  * @return a fresh allocated split object
  * @ingroup split
@@ -97,6 +97,32 @@ void elektraSplitDel(Split *keysets)
 	elektraFree (keysets);
 }
 
+/**
+ * @brief Remove one part of split.
+ *
+ * @param split the split object to work with
+ * @param where the position to cut away
+ *
+ * @pre where must be within the size of the split
+ * @post split will be removed
+ *
+ * @ingroup split
+ */
+void elektraSplitRemove(Split *split, size_t where)
+{
+	ELEKTRA_ASSERT(where < split->size && "cannot remove behind size");
+	ksDel(split->keysets[where]);
+	keyDel(split->parents[where]);
+	--split->size; // reduce size
+	for(size_t i=where; i<split->size; ++i)
+	{
+		split->keysets[i] = split->keysets[i+1];
+		split->handles[i] = split->handles[i+1];
+		split->parents[i] = split->parents[i+1];
+		split->syncbits[i] = split->syncbits[i+1];
+	}
+}
+
 /**
  * Doubles the size of how many parts of keysets can be appended.
  *
@@ -127,7 +153,7 @@ void elektraSplitResize(Split *split)
  * @param syncbits the initial syncstate which should be appended
  * @ingroup split
  * @retval -1 if no split is found
- * @return the size of split - 1
+ * @return the position of the new element: size-1
  */
 ssize_t elektraSplitAppend(Split *split, Backend *backend, Key *parentKey, int syncbits)
 {
@@ -141,12 +167,15 @@ ssize_t elektraSplitAppend(Split *split, Backend *backend, Key *parentKey, int s
 	++ split->size;
 	if (split->size > split->alloc) elektraSplitResize(split);
 
-	split->keysets[split->size-1]=ksNew(0, KS_END);
-	split->handles[split->size-1]=backend;
-	split->parents[split->size-1]=parentKey;
-	split->syncbits[split->size-1]=syncbits;
+	// index of the new element
+	const int n = split->size-1;
 
-	return split->size-1;
+	split->keysets[n]=ksNew(0, KS_END);
+	split->handles[n]=backend;
+	split->parents[n]=parentKey;
+	split->syncbits[n]=syncbits;
+
+	return n;
 }
 
 /**
@@ -164,17 +193,23 @@ ssize_t elektraSplitAppend(Split *split, Backend *backend, Key *parentKey, int s
  */
 ssize_t elektraSplitSearchBackend(Split *split, Backend *backend, Key *parent)
 {
-	/* TODO: possible optimization: use an index to find already inserted backends */
 	for (size_t i=0; i<split->size; ++i)
 	{
 		if (backend == split->handles[i])
 		{
-			if (split->syncbits[i] & 2)
+			if (test_bit(split->syncbits[i], SPLIT_FLAG_CASCADING))
 			{
-				if ((keyIsUser(parent) == 1 && keyIsUser(split->parents[i]) == 1)  ||
-				    (keyIsSystem(parent) == 1 && keyIsSystem(split->parents[i]) == 1))
+				switch (keyGetNamespace(parent))
 				{
-					return i;
+				case KEY_NS_SPEC: if (keyIsSpec(split->parents[i])) return i; break;
+				case KEY_NS_DIR: if (keyIsDir(split->parents[i])) return i; break;
+				case KEY_NS_USER: if (keyIsUser(split->parents[i])) return i; break;
+				case KEY_NS_SYSTEM: if (keyIsSystem(split->parents[i])) return i; break;
+				case KEY_NS_PROC: return -1;
+				case KEY_NS_EMPTY: return -1;
+				case KEY_NS_NONE: return -1;
+				case KEY_NS_META: return -1;
+				case KEY_NS_CASCADING: return -1;
 				}
 				continue;
 			}
@@ -198,11 +233,9 @@ ssize_t elektraSplitSearchBackend(Split *split, Backend *backend, Key *parent)
  */
 int elektraSplitSearchRoot(Split *split, Key *parentKey)
 {
-	size_t splitSize = split->size;
-
 	if (!parentKey) return 0;
 
-	for (size_t i=0; i<splitSize; ++i)
+	for (size_t i=0; i<split->size; ++i)
 	{
 		if (keyRel (split->parents[i], parentKey) >= 0)
 			return 1;
@@ -213,9 +246,48 @@ int elektraSplitSearchRoot(Split *split, Key *parentKey)
 
 
 /**
- * Walks through the trie and adds all backends below parentKey.
+ * @brief Map namespace to string and decide if it should be used for kdbGet()
+ *
+ * @param parentKey the key name that should be changed
+ * @param ns the namespace it should be changed to
+ *
+ * @retval 0 invalid namespace for kdbGet(), no action required
+ * @retval 1 valid namespace for kdbGet()
+ */
+static int elektraKeySetNameByNamespace(Key *parentKey, elektraNamespace ns)
+{
+	switch (ns)
+	{
+	case KEY_NS_SPEC:
+		keySetName(parentKey, "spec");
+		break;
+	case KEY_NS_PROC:
+		/* only transient, should fail */
+		return 0;
+	case KEY_NS_DIR:
+		keySetName(parentKey, "dir");
+		break;
+	case KEY_NS_USER:
+		keySetName(parentKey, "user");
+		break;
+	case KEY_NS_SYSTEM:
+		keySetName(parentKey, "system");
+		break;
+	case KEY_NS_EMPTY:
+	case KEY_NS_NONE:
+	case KEY_NS_META:
+	case KEY_NS_CASCADING:
+		return 0;
+	}
+	return 1;
+}
+
+
+/**
+ * Walks through kdb->split and adds all backends below parentKey to split.
  *
  * Sets syncbits to 2 if it is a default or root backend (which needs splitting).
+ * The information is copied from kdb->split.
  *
  * @pre split needs to be empty, directly after creation with elektraSplitNew().
  *
@@ -225,7 +297,7 @@ int elektraSplitSearchRoot(Split *split, Key *parentKey)
  * @pre parentKey must be a valid key! (could be implemented more generally,
  *      but that would require splitting up of keysets of the same backend)
  *
- * @param split the split object to work with
+ * @param split will get all backends appended
  * @param kdb the handle to get information about backends
  * @param parentKey the information below which key the backends are from interest
  * @ingroup split
@@ -233,33 +305,65 @@ int elektraSplitSearchRoot(Split *split, Key *parentKey)
  */
 int elektraSplitBuildup (Split *split, KDB *kdb, Key *parentKey)
 {
-
-	if (!parentKey || !strcmp(keyName(parentKey), ""))
+	/* For compatibility reasons invalid names are accepted, too.
+	 * This solution is faster than checking the name of parentKey
+	 * every time in loop.
+	 * The parentKey might be null in some unit tests, so also check
+	 * for this. */
+	const char *name = keyName(parentKey);
+	if (!parentKey || !name || !strcmp(name, "") || !strcmp(name, "/"))
 	{
 		parentKey = 0;
 	}
+	else if (name[0] == '/')
+	{
+		Key *key = keyNew(0, KEY_END);
+		for (elektraNamespace ins=KEY_NS_FIRST; ins<=KEY_NS_LAST; ++ins)
+		{
+			if (!elektraKeySetNameByNamespace(key, ins)) continue;
+			keyAddName(key, keyName(parentKey));
+			elektraSplitBuildup(split, kdb, key);
+		}
+		keyDel (key);
+		return 1;
+	}
 
 	/* Returns the backend the key is in or the default backend
 	   otherwise */
 	Backend * backend = elektraMountGetBackend(kdb, parentKey);
 
+#if DEBUG && VERBOSE
+	printf (" with parent %s\n", keyName(parentKey));
+#endif
 	for (size_t i=0; i < kdb->split->size; ++i)
 	{
-		if (backend == kdb->split->handles[i] && keyRel(kdb->split->parents[i], parentKey) >= 0)
+#if DEBUG && VERBOSE
+		printf ("  %d with parent %s\n", i, keyName(kdb->split->parents[i]));
+#endif
+		if (!parentKey)
 		{
+#if DEBUG && VERBOSE
+			printf ("   def add %s\n", keyName(kdb->split->parents[i]));
+#endif
+			/* Catch all: add all mountpoints */
+			elektraSplitAppend (split, kdb->split->handles[i], keyDup(kdb->split->parents[i]), kdb->split->syncbits[i]);
+		}
+		else if (backend == kdb->split->handles[i] && keyRel(kdb->split->parents[i], parentKey) >= 0)
+		{
+#if DEBUG && VERBOSE
+			printf ("   exa add %s\n", keyName(kdb->split->parents[i]));
+#endif
 			/* parentKey is exactly in this backend, so add it! */
 			elektraSplitAppend (split, kdb->split->handles[i], keyDup(kdb->split->parents[i]), kdb->split->syncbits[i]);
 		}
 		else if (keyRel(parentKey, kdb->split->parents[i]) >= 0)
 		{
+#if DEBUG && VERBOSE
+			printf ("   rel add %s\n", keyName(kdb->split->parents[i]));
+#endif
 			/* this backend is completely below the parentKey, so lets add it. */
 			elektraSplitAppend (split, kdb->split->handles[i], keyDup(kdb->split->parents[i]), kdb->split->syncbits[i]);
 		}
-		else if (parentKey == 0)
-		{
-			/* We want every backend. */
-			elektraSplitAppend (split, kdb->split->handles[i], keyDup(kdb->split->parents[i]), kdb->split->syncbits[i]);
-		}
 	}
 
 	return 1;
@@ -268,10 +372,10 @@ int elektraSplitBuildup (Split *split, KDB *kdb, Key *parentKey)
 
 
 /**
- * Splits up the keysets and search for a sync bit.
+ * Splits up the keysets and search for a sync bit in every key.
  *
  * It does not check if there were removed keys,
- * see elektraSplitRemove() for the next step.
+ * see elektraSplitSync() for the next step.
  *
  * It does not create new backends, this has to be
  * done by buildup before.
@@ -284,7 +388,7 @@ int elektraSplitBuildup (Split *split, KDB *kdb, Key *parentKey)
  *
  * @return 0 if there were no sync bits
  * @return 1 if there were sync bits
- * @return -1 if no backend was found for a key
+ * @return -1 if no backend was found for any key
  * @ingroup split
  */
 int elektraSplitDivide (Split *split, KDB *handle, KeySet *ks)
@@ -297,12 +401,13 @@ int elektraSplitDivide (Split *split, KDB *handle, KeySet *ks)
 	ksRewind (ks);
 	while ((curKey = ksNext (ks)) != 0)
 	{
+		// TODO: handle keys in wrong namespaces
 		curHandle = elektraMountGetBackend(handle, curKey);
 		if (!curHandle) return -1;
 
 		curFound = elektraSplitSearchBackend(split, curHandle, curKey);
 
-		if (curFound == -1) continue;
+		if (curFound == -1) continue; // key not relevant in this kdbSet
 
 		ksAppendKey (split->keysets[curFound], curKey);
 		if (keyNeedSync(curKey) == 1)
@@ -315,6 +420,27 @@ int elektraSplitDivide (Split *split, KDB *handle, KeySet *ks)
 	return needsSync;
 }
 
+/**
+ * @brief Update the (configuration) file name for the parent key
+ *
+ * @param split the split to work with
+ * @param handle the handle to work with
+ * @param key the parentKey that should be updated (name must be
+ * correct)
+ */
+void elektraSplitUpdateFileName (Split *split, KDB *handle, Key *key)
+{
+	Backend *curHandle = elektraMountGetBackend(handle, key);
+	if (!curHandle) return;
+	ssize_t curFound = elektraSplitSearchBackend(split, curHandle, key);
+	if (curFound == -1) return;
+#if DEBUG && VERBOSE
+	printf ("Update string from %s to %s\n", keyString(key), keyString(split->parents[curFound]));
+	printf ("Names are: %s and %s\n\n", keyName(key), keyName(split->parents[curFound]));
+#endif
+	keySetString(key, keyString(split->parents[curFound]));
+}
+
 
 /**
  * Appoints all keys from ks to yet unsynced splits.
@@ -346,7 +472,7 @@ int elektraSplitAppoint (Split *split, KDB *handle, KeySet *ks)
 
 		if (curFound == -1) curFound = defFound;
 
-		if (split->syncbits[curFound] & 1)
+		if (split->syncbits[curFound] & SPLIT_FLAG_SYNC)
 		{
 			continue;
 		}
@@ -372,7 +498,9 @@ static void elektraDropCurrentKey(KeySet *ks,
 			strlen(msg) +
 			sizeOfStaticText);
 	strcpy(warningMsg, "drop key ");
-	strcat(warningMsg, keyName(k));
+	const char *name = keyName(k);
+	if (name) strcat(warningMsg, name);
+	else strcat(warningMsg, "(no name)");
 	strcat(warningMsg, " not belonging to ");
 	strcat(warningMsg, keyName(curHandle->mountpoint));
 	strcat(warningMsg, " with name ");
@@ -387,81 +515,161 @@ static void elektraDropCurrentKey(KeySet *ks,
 	ksPrev(ks); // next ksNext() will point correctly again
 }
 
+
+/**
+ * @brief Filter out keys not in the correct keyset
+ *
+ * @param split the split where to do it
+ * @param i for which split
+ * @param warningKey the key
+ * @param handle where to do backend lookups
+ *
+ * @retval -1 on error (no backend, wrong namespace)
+ * @retval 0 otherwise
+ */
+static int elektraSplitPostprocess (Split *split, int i, Key *warningKey, KDB *handle)
+{
+	Key *cur = 0;
+	Backend *curHandle = 0;
+	ksRewind (split->keysets[i]);
+	while ((cur = ksNext(split->keysets[i])) != 0)
+	{
+		curHandle = elektraMountGetBackend(handle, cur);
+		if (!curHandle) return -1;
+
+		keyClearSync (cur);
+
+		if (curHandle != split->handles[i])
+		{
+			elektraDropCurrentKey(split->keysets[i], warningKey, curHandle, "it is hidden by other mountpoint");
+		}
+		else switch (keyGetNamespace(cur))
+		{
+		case KEY_NS_SPEC:
+			if (!keyIsSpec(split->parents[i]))
+				elektraDropCurrentKey(split->keysets[i], warningKey, curHandle, "it is not spec");
+			break;
+		case KEY_NS_DIR:
+			if (!keyIsDir(split->parents[i]))
+				elektraDropCurrentKey(split->keysets[i], warningKey, curHandle, "it is not dir");
+			break;
+		case KEY_NS_USER:
+			if (!keyIsUser(split->parents[i]))
+				elektraDropCurrentKey(split->keysets[i], warningKey, curHandle, "it is not user");
+			break;
+		case KEY_NS_SYSTEM:
+			if (!keyIsSystem(split->parents[i]))
+				elektraDropCurrentKey(split->keysets[i], warningKey, curHandle, "it is not system");
+			break;
+		case KEY_NS_PROC:
+			elektraDropCurrentKey(split->keysets[i], warningKey, curHandle, "it has a proc key name");
+			break;
+		case KEY_NS_EMPTY:
+			elektraDropCurrentKey(split->keysets[i], warningKey, curHandle, "it has an empty name");
+			break;
+		case KEY_NS_META:
+			elektraDropCurrentKey(split->keysets[i], warningKey, curHandle, "it has a meta name");
+			break;
+		case KEY_NS_CASCADING:
+			elektraDropCurrentKey(split->keysets[i], warningKey, curHandle, "it has a cascading name");
+			break;
+		case KEY_NS_NONE:
+			ELEKTRA_ASSERT(0 && "wrong key namespace, should not be none");
+			return -1;
+		}
+	}
+	return 0;
+}
+
+
 /**
  * Does some work after getting of backends is finished.
  *
+ * - Update sizes
+ * - Removal of wrong keys
+ *
  * @pre elektraSplitAppoint() needs to be executed before.
  *
  * - check if keys are in correct backend
  * - remove syncbits
- * - update usersize and systemsize
+ * - update sizes in the backends
  *
  * @param split the split object to work with
  * @param warningKey postcondition violations are reported here
  * @param handle the handle to preprocess the keys
  * @return 1 on success
- * @return -1 if no backend was found for a key
+ * @return -1 if no backend was found for a key or split->parents
+ *         has invalid namespace
  * @ingroup split
  */
 int elektraSplitGet (Split *split, Key *warningKey, KDB *handle)
 {
-	Key *cur = 0;
-	Backend *curHandle = 0;
-
+	int ret = 1;
 	/* Dont iterate the default split part */
-	for (size_t i=0; i<split->size-1; ++i)
+	const int bypassedSplits = 1;
+
+	for (size_t i=0; i<split->size-bypassedSplits; ++i)
 	{
-		if (!(split->syncbits[i] & 1))
+		if (test_bit(split->syncbits[i], 0))
 		{
 			/* Dont process keysets which come from the user
 			   and not from the backends */
 			continue;
 		}
 
-		ksRewind (split->keysets[i]);
-		while ((cur = ksNext(split->keysets[i])) != 0)
-		{
-			curHandle = elektraMountGetBackend(handle, cur);
-			if (!curHandle) return -1;
-
-			if (curHandle != split->handles[i])
-			{
-				elektraDropCurrentKey(split->keysets[i], warningKey, curHandle, "it is hidden by other mountpoint");
-				continue;
-			}
-			if (keyGetNameSize(cur) == 0)
-			{
-				elektraDropCurrentKey(split->keysets[i], warningKey, curHandle, "it has an empty name");
-				continue;
-			}
-			if (!strncmp(keyName(cur), "user", 4) && strncmp(keyName(split->parents[i]), "user", 4))
-			{
-				elektraDropCurrentKey(split->keysets[i], warningKey, curHandle, "it is not user");
-				continue;
-			}
-			if (!strncmp(keyName(cur), "system", 6) && strncmp(keyName(split->parents[i]), "system", 6))
-			{
-				elektraDropCurrentKey(split->keysets[i], warningKey, curHandle, "it is not system");
-				continue;
-			}
-
-			keyClearSync (cur);
-		}
-
 		/* Update sizes */
-		if (!strncmp(keyName(split->parents[i]), "system", 6))
-		{
-			split->handles[i]->systemsize = ksGetSize(split->keysets[i]);
-		}
-		else if (!strncmp(keyName(split->parents[i]), "user", 4))
-		{
-			split->handles[i]->usersize = ksGetSize(split->keysets[i]);
-		}
+#if DEBUG && VERBOSE
+		printf ("Update size for %s\n", keyName(split->parents[i]));
+#endif
+		// first we need postprocessing because that might
+		// reduce sizes
+		if (elektraSplitPostprocess(split, i, warningKey, handle) == -1) ret = -1;
+		// then we can set the size
+		if (elektraBackendUpdateSize(split->handles[i], split->parents[i], ksGetSize(split->keysets[i])) == -1) ret = -1;
 	}
 
+	return ret;
+}
+
+/**
+ * @brief Check if any of the split is uninitialized
+ *
+ * @param split
+ *
+ * @retval -1 if size is wrong
+ * @retval 1 if everything is ok
+ */
+int elektraSplitCheckSize (Split *split)
+{
+	/* Iterate everything */
+	for (size_t i=0; i<split->size; ++i)
+	{
+		switch (keyGetNamespace(split->parents[i]))
+		{
+		case KEY_NS_SPEC:
+			if (split->handles[i]->specsize == -1) return -1;
+			break;
+		case KEY_NS_DIR:
+			if (split->handles[i]->dirsize == -1) return -1;
+			break;
+		case KEY_NS_USER:
+			if (split->handles[i]->usersize == -1) return -1;
+			break;
+		case KEY_NS_SYSTEM:
+			if (split->handles[i]->systemsize == -1) return -1;
+			break;
+		case KEY_NS_PROC:
+		case KEY_NS_EMPTY:
+		case KEY_NS_NONE:
+		case KEY_NS_META:
+		case KEY_NS_CASCADING:
+			return -1;
+		}
+	}
 	return 1;
 }
 
+
 /**
  * Also update sizes after kdbSet() to recognize multiple kdbSet() attempts.
  *
@@ -473,13 +681,26 @@ int elektraSplitUpdateSize (Split *split)
 	/* Iterate everything */
 	for (size_t i=0; i<split->size; ++i)
 	{
-		if (!strncmp(keyName(split->parents[i]), "system", 6))
-		{
-			split->handles[i]->systemsize = ksGetSize(split->keysets[i]);
-		}
-		else if (!strncmp(keyName(split->parents[i]), "user", 4))
+		switch (keyGetNamespace(split->parents[i]))
 		{
+		case KEY_NS_SPEC:
+			split->handles[i]->specsize = ksGetSize(split->keysets[i]);
+			break;
+		case KEY_NS_DIR:
+			split->handles[i]->dirsize = ksGetSize(split->keysets[i]);
+			break;
+		case KEY_NS_USER:
 			split->handles[i]->usersize = ksGetSize(split->keysets[i]);
+			break;
+		case KEY_NS_SYSTEM:
+			split->handles[i]->systemsize = ksGetSize(split->keysets[i]);
+			break;
+		case KEY_NS_PROC:
+		case KEY_NS_EMPTY:
+		case KEY_NS_NONE:
+		case KEY_NS_META:
+		case KEY_NS_CASCADING:
+			return -1;
 		}
 	}
 	return 1;
@@ -503,14 +724,23 @@ int elektraSplitMerge (Split *split, KeySet *dest)
 	return 1;
 }
 
-/** Add sync bits everywhere keys were removed.
+/** Add sync bits everywhere keys were removed/added.
  *
- * Only this function can really decide if sync is needed or not.
+ * - checks if the size of a previous kdbGet() is unchanged.
+ * - checks if in correct state (kdbGet() needs to be executed before)
+ *
+ * Only elektraSplitDivide() together with this function can really decide
+ * if sync is needed or not.
  *
  * @pre split needs to be processed with elektraSplitDivide() before.
  *
- * @return 0 if kdbSet() is not needed
- * @return 1 if kdbSet() is needed
+ * @retval 0 if kdbSet() is not needed
+ * @retval 1 if kdbSet() is needed
+ * @retval -1 on wrong keys (also has assert, should not happen)
+ * @retval -2 wrong spec state: kdbGet() was not executed before
+ * @retval -3 wrong dir state: kdbGet() was not executed before
+ * @retval -4 wrong user state: kdbGet() was not executed before
+ * @retval -5 wrong system state: kdbGet() was not executed before
  * @pre user/system was split before.
  * @param split the split object to work with
  * @ingroup split
@@ -522,27 +752,68 @@ int elektraSplitSync(Split *split)
 
 	for (size_t i=0; i<split->size; ++i)
 	{
-		if (split->syncbits[i] & 1)
+		// first check for wrong states etc.
+		switch (keyGetNamespace(split->parents[i]))
 		{
-			needsSync = 1;
-			continue;
-		}
-
-		if (!strncmp(keyName(split->parents[i]), "system", 6))
-		{
-			/* Check for system keyset for removed keys */
-			if (split->handles[i]->systemsize != ksGetSize(split->keysets[i]))
+		case KEY_NS_SPEC:
+			// Check if we are in correct state
+			if (split->handles[i]->specsize == -1)
 			{
-				split->syncbits[i] |= 1;
+				return -i-2;
+			}
+			/* Check for spec keyset for removed keys */
+			if (split->handles[i]->specsize != ksGetSize(split->keysets[i]))
+			{
+				set_bit(split->syncbits[i], SPLIT_FLAG_SYNC);
 				needsSync = 1;
 			}
-		} else if (!strncmp(keyName(split->parents[i]), "user", 4)) {
+			break;
+		case KEY_NS_DIR:
+			// Check if we are in correct state
+			if (split->handles[i]->dirsize == -1)
+			{
+				return -i-2;
+			}
+			/* Check for dir keyset for removed keys */
+			if (split->handles[i]->dirsize != ksGetSize(split->keysets[i]))
+			{
+				set_bit(split->syncbits[i], SPLIT_FLAG_SYNC);
+				needsSync = 1;
+			}
+			break;
+		case KEY_NS_USER:
+			// Check if we are in correct state
+			if (split->handles[i]->usersize == -1)
+			{
+				return -i-2;
+			}
 			/* Check for user keyset for removed keys */
 			if (split->handles[i]->usersize != ksGetSize(split->keysets[i]))
 			{
-				split->syncbits[i] |= 1;
+				set_bit(split->syncbits[i], SPLIT_FLAG_SYNC);
 				needsSync = 1;
 			}
+			break;
+		case KEY_NS_SYSTEM:
+			// Check if we are in correct state
+			if (split->handles[i]->systemsize == -1)
+			{
+				return -i-2;
+			}
+			/* Check for system keyset for removed keys */
+			if (split->handles[i]->systemsize != ksGetSize(split->keysets[i]))
+			{
+				set_bit(split->syncbits[i], SPLIT_FLAG_SYNC);
+				needsSync = 1;
+			}
+			break;
+		case KEY_NS_PROC:
+		case KEY_NS_EMPTY:
+		case KEY_NS_META:
+		case KEY_NS_CASCADING:
+		case KEY_NS_NONE:
+			ELEKTRA_ASSERT(0 && "Got keys that should not be here");
+			return -1;
 		}
 	}
 
@@ -556,71 +827,22 @@ int elektraSplitSync(Split *split)
  *
  * @param split the split object to work with
  * @ingroup split
- * @retval 0 on success
  */
-int elektraSplitPrepare (Split *split)
+void elektraSplitPrepare (Split *split)
 {
-	size_t size = split->size;
-	for (size_t i=0; i<size; ++i)
+	for (size_t i=0; i<split->size; ++i)
 	{
 		if ((split->syncbits[i] & 1) == 1)
 		{
 			KeySet *n = ksDeepDup(split->keysets[i]);
 			ksDel (split->keysets[i]);
 			split->keysets[i] = n;
-			continue;
 		}
-
-		if ((split->syncbits[i] & 1) == 0)
+		else
 		{
 			/* We don't need i anymore */
-			for (size_t j = i+1; j<size; ++j)
-			{
-				if ((split->syncbits[j] & 1) == 1)
-				{
-					Backend *tmpBackend = 0;
-					Key *tmpParent = 0;
-					int tmpSyncbits = 0;
-
-					/* Ohh, we have found an important j... let's swap j and i */
-					ksDel (split->keysets[i]);
-					split->keysets[i] = ksDup(split->keysets[j]);
-
-					tmpBackend = split->handles[i];
-					split->handles[i] = split->handles[j];
-					split->handles[j] = tmpBackend;
-
-					tmpParent = split->parents[i];
-					split->parents[i] = split->parents[j];
-					split->parents[j] = tmpParent;
-
-					tmpSyncbits = split->syncbits[i];
-					split->syncbits[i] = split->syncbits[j];
-					split->syncbits[j] = tmpSyncbits;
-					goto cont; /* Continue the outer loop */
-				}
-			}
-
-			/* We are finished, search did not find anything which needed sync, so let's remove the rest */
-			for (size_t j = i; j<size; ++j)
-			{
-				if ((split->syncbits[j] & 1) == 0)
-				{
-					ksDel (split->keysets[j]);
-					keyDecRef (split->parents[j]);
-					keyDel (split->parents[j]);
-
-					split->keysets[j]=0;
-					split->handles[j]=0;
-					split->parents[j]=0;
-					split->syncbits[j]=0;
-					--split->size;
-				}
-			}
-			break;
+			elektraSplitRemove(split, i);
+			--i;
 		}
-cont: ;
 	}
-
-	return 0;
 }
