@@ -77,20 +77,7 @@ int elektraSyslogGet(Plugin *handle ELEKTRA_UNUSED, KeySet *returned, Key *paren
 		keyNew ("system/elektra/modules/syslog/exports/error",
 			KEY_FUNC, elektraSyslogError,
 			KEY_END),
-		keyNew ("system/elektra/modules/syslog/infos",
-			KEY_VALUE, "All information you want to know", KEY_END),
-		keyNew ("system/elektra/modules/syslog/infos/author",
-			KEY_VALUE, "Markus Raab <elektra@markus-raab.org>", KEY_END),
-		keyNew ("system/elektra/modules/syslog/infos/licence",
-			KEY_VALUE, "BSD", KEY_END),
-		keyNew ("system/elektra/modules/syslog/infos/description",
-			KEY_VALUE, "Logs set and error calls to syslog", KEY_END),
-		keyNew ("system/elektra/modules/syslog/infos/provides",
-			KEY_VALUE, "logging", KEY_END),
-		keyNew ("system/elektra/modules/syslog/infos/placements",
-			KEY_VALUE, "postcommit postrollback", KEY_END),
-		keyNew ("system/elektra/modules/syslog/infos/needs",
-			KEY_VALUE, "", KEY_END),
+#include "readme_syslog.c"
 		keyNew ("system/elektra/modules/syslog/infos/version",
 			KEY_VALUE, PLUGINVERSION, KEY_END),
 		KS_END));
