@@ -36,7 +36,7 @@ int ImportCommand::execute(Cmdline const& cl)
 
 	KeySet originalKeys;
 	kdb.get (originalKeys, root);
-	KeySet existingKeys = originalKeys.cut (root);
+	KeySet base = originalKeys.cut (root);
 	printWarnings (cerr, root);
 
 	KeySet importedKeys;
@@ -61,20 +61,9 @@ int ImportCommand::execute(Cmdline const& cl)
 	ThreeWayMerge merger;
 	MergeHelper helper;
 
-	KeySet base;
-
-	// TODO: this should not be neccessary, but
-	// applying threeway strategies to a twoway merge is hard
-	base = KeySet();
-
-	// TODO: for now we have to position this strategy manually
-	// to avoid meta information loss
-	MetaMergeStrategy metaStrategy(merger);
-	merger.addConflictStrategy(&metaStrategy);
-
-	helper.parseStrategies (cl, merger);
+	helper.configureMerger (cl, merger);
 	MergeResult result = merger.mergeKeySet (
-			MergeTask (BaseMergeKeys (base, root), OurMergeKeys (existingKeys, root),
+			MergeTask (BaseMergeKeys (base, root), OurMergeKeys (base, root),
 					TheirMergeKeys (importedKeys, root), root));
 
 	helper.reportResult (cl, result, cout, cerr);
