@@ -57,6 +57,7 @@ void MountCommand::outputMtab(Cmdline const& cl)
 			else std::cout << delim << std::flush;
 		}
 
+		// TODO: remove next version
 		if (cl.third)
 		{
 			std::cout << it->name;
@@ -86,7 +87,16 @@ void MountCommand::buildBackend(Cmdline const& cl)
 {
 	Backend backend;
 
-	backend.setMountpoint(Key(mp, KEY_CASCADING_NAME, KEY_END), mountConf);
+	Key mpk(mp, KEY_CASCADING_NAME, KEY_END);
+
+	if (!mpk.isValid())
+	{
+		throw invalid_argument(mp + " is not a valid mountpoint");
+	}
+
+	backend.setMountpoint(mpk, mountConf);
+
+	backend.setBackendConfig(cl.getPluginsConfig("system/"));
 
 	if (cl.debug)
 	{
@@ -111,19 +121,7 @@ void MountCommand::buildBackend(Cmdline const& cl)
 		path = cl.arguments[0];
 	}
 
-	backend.checkFile (path);
-
-	std::string configPath = Backends::getConfigBasePath(name);
-
-	mountConf.append ( *Key(configPath,
-			KEY_VALUE, "",
-			KEY_COMMENT, "This is a configuration for a backend, see subkeys for more information",
-			KEY_END));
-	configPath += "/path";
-	mountConf.append ( *Key(configPath,
-			KEY_VALUE, path.c_str(),
-			KEY_COMMENT, "The path for this backend. Note that plugins can override that with more specific configuration.",
-			KEY_END));
+	backend.useConfigFile(path);
 
 	istringstream sstream(cl.plugins);
 	std::string plugin;
@@ -144,24 +142,14 @@ void MountCommand::buildBackend(Cmdline const& cl)
 
 	appendPlugins(cl, backend);
 
-	Key rootKey (Backends::mountpointsPath, KEY_END);
-	backend.serialise (rootKey, mountConf);
+	backend.serialize(mountConf);
 }
 
-void MountCommand::addConfig (string const& configBasePath, string const& keyName, string const& value)
-{
-	Key configKey = Key (configBasePath + "/" + keyName, KEY_END);
-	// configKey.addName (keyName);
-	configKey.setString (value);
-	mountConf.append (configKey);
-}
-
-bool MountCommand::readPluginConfig(Cmdline const& cl, size_t current_token)
+bool MountCommand::readPluginConfig(Cmdline const& cl, size_t current_token, KeySet & pluginConfig)
 {
 	string keyName;
 	string value;
 
-	const string configBasePath = Backends::getConfigBasePath (name);
 	if (cl.interactive)
 	{
 		cout << "Enter the plugin configuration" << endl;
@@ -177,7 +165,7 @@ bool MountCommand::readPluginConfig(Cmdline const& cl, size_t current_token)
 			cout << "Enter the Key value: ";
 			cin >> value;
 
-			addConfig (configBasePath, keyName, value);
+			pluginConfig.append(Key("user/"+keyName, KEY_VALUE, value.c_str(), KEY_END));
 		}
 	}
 	else
@@ -201,7 +189,7 @@ bool MountCommand::readPluginConfig(Cmdline const& cl, size_t current_token)
 			// in the config string, consider the value empty
 			if (!std::getline (sstream, value, ',')) value = "";
 
-			addConfig (configBasePath, keyName, value);
+			pluginConfig.append(Key("user/"+keyName, KEY_VALUE, value.c_str(), KEY_END));
 		}
 	}
 
@@ -212,11 +200,12 @@ void MountCommand::appendPlugins(Cmdline const& cl, Backend & backend)
 {
 	std::string pname;
 	size_t current_plugin = 2;
+	KeySet pluginConfig;
 	if (cl.interactive)
 	{
 		cout << "First Plugin: ";
 		cin >> pname;
-		readPluginConfig (cl, current_plugin);
+		readPluginConfig (cl, current_plugin, pluginConfig);
 	}
 	else
 	{
@@ -227,7 +216,7 @@ void MountCommand::appendPlugins(Cmdline const& cl, Backend & backend)
 		else
 		{
 			pname = cl.arguments[current_plugin];
-			if (readPluginConfig (cl, current_plugin))
+			if (readPluginConfig (cl, current_plugin, pluginConfig))
 			{
 				current_plugin ++;
 			}
@@ -239,15 +228,19 @@ void MountCommand::appendPlugins(Cmdline const& cl, Backend & backend)
 	while (pname != "." || !backend.validated())
 	{
 		try {
-			KeySet pluginConfig;
-			pluginConfig.append(cl.getPluginsConfig("system/"));
-			// TODO: add per plugin config, too
 			backend.addPlugin (pname, pluginConfig);
+			pluginConfig.clear();
 		}
 		catch (PluginCheckException const& e)
 		{
-			cerr << "Could not add that plugin" << endl;
-			cerr << e.what() << endl;
+			if (!cl.interactive)
+			{
+				// do not allow errors in non-interactive mode
+				throw;
+			} else {
+				cerr << "Could not add that plugin" << endl;
+				cerr << e.what() << endl;
+			}
 		}
 		if (cl.interactive)
 		{
@@ -263,7 +256,7 @@ void MountCommand::appendPlugins(Cmdline const& cl, Backend & backend)
 
 			if (pname != ".")
 			{
-				readPluginConfig (cl, current_plugin);
+				readPluginConfig (cl, current_plugin, pluginConfig);
 			}
 		}
 		else
@@ -275,7 +268,7 @@ void MountCommand::appendPlugins(Cmdline const& cl, Backend & backend)
 			else
 			{
 				pname = cl.arguments[current_plugin];
-				if (readPluginConfig (cl, current_plugin))
+				if (readPluginConfig (cl, current_plugin, pluginConfig))
 				{
 					current_plugin ++;
 				}
@@ -312,7 +305,6 @@ int MountCommand::execute(Cmdline const& cl)
 	}
 
 	processArguments(cl);
-	fixRootKey(cl);
 	getMountpoint(cl);
 	buildBackend(cl);
 	askForConfirmation(cl);
