@@ -33,6 +33,7 @@
 #include <kdbhelper.h>
 
 #define BUFFER_LENGTH 4096
+#define ELEKTRA_TEST_ROOT "/tests/ckdb/"
 
 extern int nbError;
 extern int nbTest;
@@ -108,7 +109,7 @@ int init(int argc, char** argv);
 	{ \
 		char errorMsg [BUFFER_LENGTH]; \
 		 \
-		strcpy(errorMsg, "key name "); \
+		strcpy(errorMsg, "key value "); \
 		strcat(errorMsg, keyName(smmk1)); \
 		strcat(errorMsg, " is not equal "); \
 		strcat(errorMsg, keyName(smmk2)); \
@@ -251,6 +252,10 @@ int init(int argc, char** argv);
 			nbError++; \
 			printf("%s:%d: error in %s: Compare keyset failed, size of keysets are not equal with size(%s): %d, size(%s): %d\n", \
 				__FILE__, __LINE__, __FUNCTION__, quote_string(mmks1), (int)ksGetSize(mmks1), quote_string(mmks2), (int)ksGetSize(mmks2)); \
+			printf ("mmks1:\n"); \
+			output_keyset(mmks1); \
+			printf ("mmks2:\n"); \
+			output_keyset(mmks2); \
 		} \
 		else \
 		{ \
